import { OptionList } from '../util/Options.js';
import { InputJax } from './InputJax.js';
import { OutputJax } from './OutputJax.js';
import { MathList } from './MathList.js';
import { MathItem } from './MathItem.js';
export interface MathDocument {
    document: any;
    kind: string;
    options: OptionList;
    math: MathList;
    processed: {
        [name: string]: boolean;
    };
    inputJax: InputJax[];
    outputJax: OutputJax;
    findMath(options?: OptionList): MathDocument;
    compile(): MathDocument;
    getMetrics(): MathDocument;
    typeset(): MathDocument;
    addEventHandlers(): MathDocument;
    updateDocument(): MathDocument;
    removeFromDocument(restore?: boolean): MathDocument;
    state(state: number, restore?: boolean): MathDocument;
    reset(): MathDocument;
    clear(): MathDocument;
    concat(list: MathList): MathDocument;
}
export declare type MathProcessed = {
    findMath: boolean;
    compile: boolean;
    getMetrics: boolean;
    typeset: boolean;
    addEventHandlers: boolean;
    updateDocument: boolean;
    [name: string]: boolean;
};
export declare abstract class AbstractMathDocument implements MathDocument {
    static KIND: string;
    static OPTIONS: OptionList;
    static STATE: {
        UNPROCESSED: number;
        COMPILED: number;
        TYPESET: number;
        INSERTED: number;
    };
    document: any;
    options: OptionList;
    math: MathList;
    processed: MathProcessed;
    inputJax: InputJax[];
    outputJax: OutputJax;
    constructor(document: any, options: OptionList);
    readonly kind: string;
    findMath(options?: OptionList): this;
    compile(): this;
    compileError(math: MathItem, err: Error): void;
    typeset(): this;
    typesetError(math: MathItem, err: Error): void;
    getMetrics(): this;
    addEventHandlers(): this;
    updateDocument(): this;
    removeFromDocument(restore?: boolean): this;
    state(state: number, restore?: boolean): this;
    reset(): this;
    clear(): this;
    concat(list: MathList): this;
}
