import { MathDocument } from './MathDocument.js';
import { InputJax } from './InputJax.js';
import { OptionList } from '../util/Options.js';
import { MmlNode } from './MmlTree/MmlNode.js';
export declare type Location = {
    i?: number;
    n?: number;
    delim?: string;
    node?: Text | Element;
};
export declare type Metrics = {
    em: number;
    ex: number;
    containerWidth: number;
    lineWidth: number;
    scale: number;
};
export declare type BBox = {};
export interface MathItem {
    math: string;
    inputJax: InputJax;
    display: boolean;
    start: Location;
    end: Location;
    root: MmlNode;
    typesetRoot: Element;
    metrics: Metrics;
    bbox: BBox;
    inputData: OptionList;
    outputData: OptionList;
    compile(document: MathDocument): void;
    typeset(document: MathDocument): void;
    addEventHandlers(): void;
    updateDocument(document: MathDocument): void;
    removeFromDocument(restore: boolean): void;
    setMetrics(em: number, ex: number, cwidth: number, lwidth: number, scale: number): void;
    state(state?: number, restore?: boolean): number;
}
export declare type ProtoItem = {
    math: string;
    start: Location;
    end: Location;
    open?: string;
    close?: string;
    n?: number;
    display: boolean;
};
export declare function protoItem(open: string, math: string, close: string, n: number, start: number, end: number, display?: boolean): ProtoItem;
export declare abstract class AbstractMathItem implements MathItem {
    static STATE: {
        UNPROCESSED: number;
        COMPILED: number;
        TYPESET: number;
        INSERTED: number;
    };
    math: string;
    inputJax: InputJax;
    display: boolean;
    start: Location;
    end: Location;
    root: MmlNode;
    typesetRoot: Element;
    protected _state: number;
    metrics: Metrics;
    bbox: BBox;
    inputData: OptionList;
    outputData: OptionList;
    constructor(math: string, jax: InputJax, display?: boolean, start?: Location, end?: Location);
    compile(document: MathDocument): void;
    typeset(document: MathDocument): void;
    addEventHandlers(): void;
    updateDocument(document: MathDocument): void;
    removeFromDocument(restore?: boolean): void;
    setMetrics(em: number, ex: number, cwidth: number, lwidth: number, scale: number): void;
    state(state?: number, restore?: boolean): number;
}
