"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var sm = require("./SymbolMap.js");
var TexConstants_js_1 = require("./TexConstants.js");
var MmlNode_js_1 = require("../../core/MmlTree/MmlNode.js");
var BaseMethods_js_1 = require("./BaseMethods.js");
var ParserUtil_js_1 = require("./ParserUtil.js");
var AmsMappings;
(function (AmsMappings) {
    var COLS = function (W) {
        var WW = [];
        for (var i = 0, m = W.length; i < m; i++) {
            WW[i] = ParserUtil_js_1.ParserUtil.Em(W[i]);
        }
        return WW.join(" ");
    };
    sm.CharacterMap.create('AMSmath-mathchar0mo', BaseMethods_js_1.BaseMethods.mathchar0mo, {
        iiiint: ['2A0C', { texClass: MmlNode_js_1.TEXCLASS.OP }]
    });
    sm.CommandMap.create('AMSmath-macros', {
        mathring: ['Accent', '2DA'],
        nobreakspace: 'Tilde',
        negmedspace: ['Spacer', TexConstants_js_1.TexConstant.Length.NEGATIVEMEDIUMMATHSPACE],
        negthickspace: ['Spacer', TexConstants_js_1.TexConstant.Length.NEGATIVETHICKMATHSPACE],
        idotsint: ['MultiIntegral', '\\int\\cdots\\int'],
        dddot: ['Accent', '20DB'],
        ddddot: ['Accent', '20DC'],
        sideset: ['Macro', '\\mathop{\\mathop{\\rlap{\\phantom{#3}}}\\nolimits#1\\!\\mathop{#3}\\nolimits#2}', 3],
        boxed: ['Macro', '\\fbox{$\\displaystyle{#1}$}', 1],
        tag: 'HandleTag',
        notag: 'HandleNoTag',
        label: 'HandleLabel',
        ref: 'HandleRef',
        eqref: ['HandleRef', true],
        substack: ['Macro', '\\begin{subarray}{c}#1\\end{subarray}', 1],
        injlim: ['NamedOp', 'inj&thinsp;lim'],
        projlim: ['NamedOp', 'proj&thinsp;lim'],
        varliminf: ['Macro', '\\mathop{\\underline{\\mmlToken{mi}{lim}}}'],
        varlimsup: ['Macro', '\\mathop{\\overline{\\mmlToken{mi}{lim}}}'],
        varinjlim: ['Macro', '\\mathop{\\underrightarrow{\\mmlToken{mi}{lim}}}'],
        varprojlim: ['Macro', '\\mathop{\\underleftarrow{\\mmlToken{mi}{lim}}}'],
        DeclareMathOperator: 'HandleDeclareOp',
        operatorname: 'HandleOperatorName',
        SkipLimits: 'SkipLimits',
        genfrac: 'Genfrac',
        frac: ['Genfrac', "", "", "", ""],
        tfrac: ['Genfrac', "", "", "", 1],
        dfrac: ['Genfrac', "", "", "", 0],
        binom: ['Genfrac', "(", ")", "0", ""],
        tbinom: ['Genfrac', "(", ")", "0", 1],
        dbinom: ['Genfrac', "(", ")", "0", 0],
        cfrac: 'CFrac',
        shoveleft: ['HandleShove', TexConstants_js_1.TexConstant.Align.LEFT],
        shoveright: ['HandleShove', TexConstants_js_1.TexConstant.Align.RIGHT],
        xrightarrow: ['xArrow', 0x2192, 5, 6],
        xleftarrow: ['xArrow', 0x2190, 7, 3]
    });
    sm.EnvironmentMap.create('AMSmath-environment', {
        align: ['AMSarray', null, true, true, 'rlrlrlrlrlrl', COLS([0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0])],
        'align*': ['AMSarray', null, false, true, 'rlrlrlrlrlrl', COLS([0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0])],
        multline: ['Multline', null, true],
        'multline*': ['Multline', null, false],
        split: ['AMSarray', null, false, false, 'rl', COLS([0])],
        gather: ['AMSarray', null, true, true, 'c'],
        'gather*': ['AMSarray', null, false, true, 'c'],
        alignat: ['AlignAt', null, true, true],
        'alignat*': ['AlignAt', null, false, true],
        alignedat: ['AlignAt', null, false, false],
        aligned: ['AlignedAMSArray', null, null, null, 'rlrlrlrlrlrl', COLS([0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0]), ".5em", 'D'],
        gathered: ['AlignedAMSArray', null, null, null, 'c', null, ".5em", 'D'],
        subarray: ['Array', null, null, null, null, COLS([0]), "0.1em", 'S', 1],
        smallmatrix: ['Array', null, null, null, 'c', COLS([1 / 3]), ".2em", 'S', 1],
        'equation': ['EquationBegin', 'Equation', true],
        'equation*': ['EquationBegin', 'EquationStar', false],
        eqnarray: ['AMSarray', null, true, true, 'rcl', "0 " + TexConstants_js_1.TexConstant.Length.THICKMATHSPACE, ".5em"],
        'eqnarray*': ['AMSarray', null, false, true, 'rcl', "0 " + TexConstants_js_1.TexConstant.Length.THICKMATHSPACE, ".5em"]
    }).parser = BaseMethods_js_1.BaseMethods.environment;
    sm.DelimiterMap.create('AMSmath-delimiter', BaseMethods_js_1.BaseMethods.delimiter, {
        '\\lvert': ['007C', { texClass: MmlNode_js_1.TEXCLASS.OPEN }],
        '\\rvert': ['007C', { texClass: MmlNode_js_1.TEXCLASS.CLOSE }],
        '\\lVert': ['2016', { texClass: MmlNode_js_1.TEXCLASS.OPEN }],
        '\\rVert': ['2016', { texClass: MmlNode_js_1.TEXCLASS.CLOSE }]
    });
    AmsMappings.CONFIGURATION = {
        delimiter: ['AMSmath-delimiter'],
        macro: ['AMSmath-mathchar0mo', 'AMSmath-macros', 'AMSmath-delimiter'],
        environment: ['AMSmath-environment']
    };
})(AmsMappings = exports.AmsMappings || (exports.AmsMappings = {}));
//# sourceMappingURL=AmsMappings.js.map