"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var TreeHelper_js_1 = require("./TreeHelper.js");
var StackItem_js_1 = require("./StackItem.js");
var Stack_js_1 = require("./Stack.js");
var MapHandler_js_1 = require("./MapHandler.js");
var TexParser_js_1 = require("./TexParser.js");
var TexError_js_1 = require("./TexError.js");
var OldParser = (function () {
    function OldParser(str, env, ParseMethods) {
        this.ParseMethods = ParseMethods;
        this.NBSP = "\u00A0";
        this.macroCount = 0;
        this.i = 0;
        this.remap = MapHandler_js_1.default.getInstance().getMap('remap');
        TreeHelper_js_1.TreeHelper.printMethod("Init (Old Parser Object)");
        this.NewParser = new TexParser_js_1.default();
        ParseMethods.NEW_PARSER = this.NewParser;
        this.string = str;
        var ENV;
        if (env) {
            ENV = {};
            for (var id in env) {
                if (env.hasOwnProperty(id)) {
                    ENV[id] = env[id];
                }
            }
        }
        this.stack = new Stack_js_1.default(ENV, !!env);
        this.Parse();
        this.Push(new StackItem_js_1.StopItem());
    }
    OldParser.prototype.Parse = function () {
        TreeHelper_js_1.TreeHelper.printMethod("Parse (Old Parser Object)");
        var c, n;
        while (this.i < this.string.length) {
            c = this.string.charAt(this.i++);
            n = c.charCodeAt(0);
            if (n >= 0xD800 && n < 0xDC00) {
                c += this.string.charAt(this.i++);
            }
            this.NewParser.parse('character', [c, this]);
        }
    };
    OldParser.prototype.Push = function (arg) {
        TreeHelper_js_1.TreeHelper.printMethod("Push (Old Parser Object)");
        this.stack.Push(arg);
    };
    OldParser.prototype.PushAll = function (args) {
        TreeHelper_js_1.TreeHelper.printMethod("PushAll (Old Parser Object)");
        for (var i = 0, m = args.length; i < m; i++) {
            this.stack.Push(args[i]);
        }
    };
    OldParser.prototype.mml = function () {
        TreeHelper_js_1.TreeHelper.printMethod("mml (Old Parser Object)");
        if (!this.stack.Top().hasType('mml')) {
            return null;
        }
        var node = this.stack.Top().data[0];
        node.setTeXclass(null);
        return node;
    };
    OldParser.prototype.mmlToken = function (token) { return token; };
    OldParser.prototype.convertDelimiter = function (c) {
        TreeHelper_js_1.TreeHelper.printMethod("convertDelimiter (Old Parser Object)");
        var symbol = this.NewParser.lookup('delimiter', c);
        return symbol ? symbol.char : null;
    };
    OldParser.prototype.trimSpaces = function (text) {
        TreeHelper_js_1.TreeHelper.printMethod("trimSpaces (Old Parser Object)");
        if (typeof (text) != 'string') {
            return text;
        }
        var TEXT = text.replace(/^\s+|\s+$/g, '');
        if (TEXT.match(/\\$/) && text.match(/ $/))
            TEXT += " ";
        return TEXT;
    };
    OldParser.prototype.nextIsSpace = function () {
        TreeHelper_js_1.TreeHelper.printMethod("nextIsSpace (Old Parser Object)");
        return this.string.charAt(this.i).match(/\s/);
    };
    OldParser.prototype.GetNext = function () {
        TreeHelper_js_1.TreeHelper.printMethod("GetNext (Old Parser Object)");
        while (this.nextIsSpace()) {
            this.i++;
        }
        return this.string.charAt(this.i);
    };
    OldParser.prototype.GetCS = function (name) {
        TreeHelper_js_1.TreeHelper.printMethod("GetCS (Old Parser Object)");
        var CS = this.string.slice(this.i).match(/^([a-z]+|.) ?/i);
        if (CS) {
            this.i += CS[1].length;
            return CS[1];
        }
        else {
            this.i++;
            return " ";
        }
    };
    OldParser.prototype.GetArgument = function (name, noneOK) {
        TreeHelper_js_1.TreeHelper.printMethod("GetArgument (Old Parser Object)");
        switch (this.GetNext()) {
            case "":
                if (!noneOK) {
                    throw new TexError_js_1.default(["MissingArgFor", "Missing argument for %1", name]);
                }
                return null;
            case '}':
                if (!noneOK) {
                    throw new TexError_js_1.default(["ExtraCloseMissingOpen",
                        "Extra close brace or missing open brace"]);
                }
                return null;
            case '\\':
                this.i++;
                return "\\" + this.GetCS();
            case '{':
                var j = ++this.i, parens = 1;
                while (this.i < this.string.length) {
                    switch (this.string.charAt(this.i++)) {
                        case '\\':
                            this.i++;
                            break;
                        case '{':
                            parens++;
                            break;
                        case '}':
                            if (--parens == 0) {
                                return this.string.slice(j, this.i - 1);
                            }
                            break;
                    }
                }
                throw new TexError_js_1.default(["MissingCloseBrace", "Missing close brace"]);
        }
        return this.string.charAt(this.i++);
    };
    OldParser.prototype.GetBrackets = function (name, def) {
        TreeHelper_js_1.TreeHelper.printMethod("GetBrackets (Old Parser Object)");
        if (this.GetNext() != '[') {
            return def;
        }
        ;
        var j = ++this.i, parens = 0;
        while (this.i < this.string.length) {
            switch (this.string.charAt(this.i++)) {
                case '{':
                    parens++;
                    break;
                case '\\':
                    this.i++;
                    break;
                case '}':
                    if (parens-- <= 0) {
                        throw new TexError_js_1.default(["ExtraCloseLooking",
                            "Extra close brace while looking for %1", "']'"]);
                    }
                    break;
                case ']':
                    if (parens == 0) {
                        return this.string.slice(j, this.i - 1);
                    }
                    break;
            }
        }
        throw new TexError_js_1.default(["MissingCloseBracket",
            "Couldn't find closing ']' for argument to %1", name]);
    };
    OldParser.prototype.GetDelimiter = function (name, braceOK) {
        TreeHelper_js_1.TreeHelper.printMethod("GetDelimiter (Old Parser Object)");
        while (this.nextIsSpace()) {
            this.i++;
        }
        var c = this.string.charAt(this.i);
        this.i++;
        if (this.i <= this.string.length) {
            if (c == "\\") {
                c += this.GetCS(name);
            }
            else if (c === "{" && braceOK) {
                this.i--;
                c = this.GetArgument(name);
            }
            if (this.NewParser.contains('delimiter', c)) {
                return this.convertDelimiter(c);
            }
        }
        throw new TexError_js_1.default(["MissingOrUnrecognizedDelim",
            "Missing or unrecognized delimiter for %1", name]);
    };
    OldParser.prototype.GetDimen = function (name) {
        TreeHelper_js_1.TreeHelper.printMethod("GetDimen (Old Parser Object)");
        if (this.nextIsSpace()) {
            this.i++;
        }
        if (this.string.charAt(this.i) == '{') {
            var dimen = this.GetArgument(name);
            if (dimen.match(/^\s*([-+]?([.,]\d+|\d+([.,]\d*)?))\s*(pt|em|ex|mu|px|mm|cm|in|pc)\s*$/)) {
                return dimen.replace(/ /g, "").replace(/,/, ".");
            }
        }
        else {
            var dimen = this.string.slice(this.i);
            var match = dimen.match(/^\s*(([-+]?([.,]\d+|\d+([.,]\d*)?))\s*(pt|em|ex|mu|px|mm|cm|in|pc)) ?/);
            if (match) {
                this.i += match[0].length;
                return match[1].replace(/ /g, "").replace(/,/, ".");
            }
        }
        throw new TexError_js_1.default(["MissingDimOrUnits",
            "Missing dimension or its units for %1", name]);
    };
    OldParser.prototype.GetUpTo = function (name, token) {
        TreeHelper_js_1.TreeHelper.printMethod("GetUpTo (Old Parser Object)");
        while (this.nextIsSpace()) {
            this.i++;
        }
        var j = this.i;
        var parens = 0;
        while (this.i < this.string.length) {
            var k = this.i;
            var c = this.string.charAt(this.i++);
            switch (c) {
                case '\\':
                    c += this.GetCS();
                    break;
                case '{':
                    parens++;
                    break;
                case '}':
                    if (parens == 0) {
                        throw new TexError_js_1.default(["ExtraCloseLooking",
                            "Extra close brace while looking for %1", token]);
                    }
                    parens--;
                    break;
            }
            if (parens == 0 && c == token) {
                return this.string.slice(j, k);
            }
        }
        throw new TexError_js_1.default(["TokenNotFoundForCommand",
            "Couldn't find %1 for %2", token, name]);
    };
    OldParser.prototype.ParseArg = function (name) {
        TreeHelper_js_1.TreeHelper.printMethod("ParseArg (Old Parser Object)");
        var object = new OldParser(this.GetArgument(name), this.stack.env, this.ParseMethods);
        return object.mml();
    };
    OldParser.prototype.ParseUpTo = function (name, token) {
        TreeHelper_js_1.TreeHelper.printMethod("ParseUpTo (Old Parser Object)");
        return new OldParser(this.GetUpTo(name, token), this.stack.env, this.ParseMethods).mml();
    };
    OldParser.prototype.InternalMath = function (text, level) {
        TreeHelper_js_1.TreeHelper.printMethod("InternalMath (Old Parser Object)");
        var def = (this.stack.env['font'] ? { mathvariant: this.stack.env['font'] } : {});
        var mml = [], i = 0, k = 0, c, node, match = '', braces = 0;
        if (text.match(/\\?[${}\\]|\\\(|\\(eq)?ref\s*\{/)) {
            while (i < text.length) {
                c = text.charAt(i++);
                if (c === '$') {
                    if (match === '$' && braces === 0) {
                        node = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [(new OldParser(text.slice(k, i - 1), {}, this.ParseMethods)).mml()], {});
                        mml.push(node);
                        match = '';
                        k = i;
                    }
                    else if (match === '') {
                        if (k < i - 1)
                            mml.push(this.InternalText(text.slice(k, i - 1), def));
                        match = '$';
                        k = i;
                    }
                }
                else if (c === '{' && match !== '') {
                    braces++;
                }
                else if (c === '}') {
                    if (match === '}' && braces === 0) {
                        TreeHelper_js_1.TreeHelper.untested(12);
                        node = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [(new OldParser(text.slice(k, i), {}, this.ParseMethods)).mml()], def);
                        mml.push(node);
                        match = '';
                        k = i;
                    }
                    else if (match !== '') {
                        if (braces)
                            braces--;
                    }
                }
                else if (c === '\\') {
                    if (match === '' && text.substr(i).match(/^(eq)?ref\s*\{/)) {
                        var len = RegExp["$&"].length;
                        if (k < i - 1)
                            mml.push(this.InternalText(text.slice(k, i - 1), def));
                        match = '}';
                        k = i - 1;
                        i += len;
                    }
                    else {
                        c = text.charAt(i++);
                        if (c === '(' && match === '') {
                            if (k < i - 2)
                                mml.push(this.InternalText(text.slice(k, i - 2), def));
                            match = ')';
                            k = i;
                        }
                        else if (c === ')' && match === ')' && braces === 0) {
                            TreeHelper_js_1.TreeHelper.untested(13);
                            node = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [(new OldParser(text.slice(k, i - 2), {}, this.ParseMethods)).mml()], {});
                            mml.push(node);
                            match = '';
                            k = i;
                        }
                        else if (c.match(/[${}\\]/) && match === '') {
                            i--;
                            text = text.substr(0, i - 1) + text.substr(i);
                        }
                    }
                }
            }
            if (match !== '')
                throw new TexError_js_1.default(["MathNotTerminated", "Math not terminated in text box"]);
        }
        if (k < text.length)
            mml.push(this.InternalText(text.slice(k), def));
        if (level != null) {
            mml = [TreeHelper_js_1.TreeHelper.createNode('mstyle', mml, { displaystyle: false, scriptlevel: level })];
        }
        else if (mml.length > 1) {
            mml = [TreeHelper_js_1.TreeHelper.createNode('mrow', mml, {})];
        }
        return mml;
    };
    OldParser.prototype.InternalText = function (text, def) {
        TreeHelper_js_1.TreeHelper.printMethod("InternalText (Old Parser Object)");
        text = text.replace(/^\s+/, this.NBSP).replace(/\s+$/, this.NBSP);
        var textNode = TreeHelper_js_1.TreeHelper.createText(text);
        return TreeHelper_js_1.TreeHelper.createNode('mtext', [], def, textNode);
    };
    OldParser.prototype.GetDelimiterArg = function (name) {
        TreeHelper_js_1.TreeHelper.printMethod('AMS-GetDelimiterArg (Old Parser Object)');
        var c = this.trimSpaces(this.GetArgument(name));
        if (c == "")
            return null;
        if (this.NewParser.contains('delimiter', c)) {
            return c;
        }
        throw new TexError_js_1.default(["MissingOrUnrecognizedDelim", "Missing or unrecognized delimiter for %1", name]);
    };
    OldParser.prototype.GetStar = function () {
        TreeHelper_js_1.TreeHelper.printMethod('AMS-GetStar (Old Parser Object)');
        var star = (this.GetNext() === "*");
        if (star) {
            this.i++;
        }
        return star;
    };
    return OldParser;
}());
exports.OldParser = OldParser;
//# sourceMappingURL=OldParser.js.map