"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var sitem = require("./StackItem.js");
var Translate_js_1 = require("./Translate.js");
var TreeHelper_js_1 = require("./TreeHelper.js");
var TexError_js_1 = require("./TexError.js");
var TexConstants_js_1 = require("./TexConstants.js");
var ParserUtil_js_1 = require("./ParserUtil.js");
var OldParser_js_1 = require("./OldParser.js");
var MmlNode_js_1 = require("../../core/MmlTree/MmlNode.js");
var ParseMethods;
(function (ParseMethods) {
    var OLD_PARSER = null;
    ParseMethods.NEW_PARSER = null;
    var PRIME = "\u2032";
    var SMARTQUOTE = "\u2019";
    var NUMBER = /^(?:[0-9]+(?:\{,\}[0-9]{3})*(?:\.[0-9]*)*|\.[0-9]+)/;
    var NBSP = "\u00A0";
    var P_HEIGHT = 1.2 / .85;
    function MmlFilterAttribute(parser, name, value) {
        return value;
    }
    ParseMethods.MmlFilterAttribute = MmlFilterAttribute;
    ;
    var MmlTokenAllow = {
        fontfamily: 1, fontsize: 1, fontweight: 1, fontstyle: 1,
        color: 1, background: 1,
        id: 1, "class": 1, href: 1, style: 1
    };
    function ControlSequence(parser, c) {
        TreeHelper_js_1.TreeHelper.printMethod("ControlSequence");
        var name = parser.GetCS();
        ParseMethods.NEW_PARSER.parse('macro', [name, this]);
    }
    ParseMethods.ControlSequence = ControlSequence;
    ;
    function csMathchar0mi(parser, mchar) {
        TreeHelper_js_1.TreeHelper.printMethod("csMathchar0mi");
        var def = mchar.attributes || { mathvariant: TexConstants_js_1.TexConstant.Variant.ITALIC };
        var textNode = TreeHelper_js_1.TreeHelper.createText(mchar.char);
        var node = TreeHelper_js_1.TreeHelper.createNode('mi', [], def, textNode);
        parser.Push(parser.mmlToken(node));
    }
    ParseMethods.csMathchar0mi = csMathchar0mi;
    ;
    function csMathchar0mo(parser, mchar) {
        TreeHelper_js_1.TreeHelper.printMethod("csMathchar0mo");
        var def = mchar.attributes || {};
        def.stretchy = false;
        var textNode = TreeHelper_js_1.TreeHelper.createText(mchar.char);
        var node = TreeHelper_js_1.TreeHelper.createNode('mo', [], def, textNode);
        parser.Push(parser.mmlToken(node));
    }
    ParseMethods.csMathchar0mo = csMathchar0mo;
    ;
    function csMathchar7(parser, mchar) {
        TreeHelper_js_1.TreeHelper.printMethod("csMathchar7");
        var def = mchar.attributes || { mathvariant: TexConstants_js_1.TexConstant.Variant.NORMAL };
        if (parser.stack.env['font']) {
            def.mathvariant = parser.stack.env['font'];
        }
        var textNode = TreeHelper_js_1.TreeHelper.createText(mchar.char);
        var node = TreeHelper_js_1.TreeHelper.createNode('mi', [], def, textNode);
        parser.Push(parser.mmlToken(node));
    }
    ParseMethods.csMathchar7 = csMathchar7;
    ;
    function csDelimiter(parser, delim) {
        TreeHelper_js_1.TreeHelper.printMethod("csDelimiter");
        var def = delim.attributes || {};
        def = Object.assign({ fence: false, stretchy: false }, def);
        var textNode = TreeHelper_js_1.TreeHelper.createText(delim.char);
        var node = TreeHelper_js_1.TreeHelper.createNode('mo', [], def, textNode);
        parser.Push(parser.mmlToken(node));
    }
    ParseMethods.csDelimiter = csDelimiter;
    ;
    function csUndefined(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("csUndefined");
        throw new TexError_js_1.default(["UndefinedControlSequence", "Undefined control sequence %1", '\\' + name]);
    }
    ParseMethods.csUndefined = csUndefined;
    ;
    function envUndefined(parser, env) {
        TreeHelper_js_1.TreeHelper.printMethod("envUndefined");
        throw new TexError_js_1.default(["UnknownEnv", "Unknown environment '%1'", env]);
    }
    ParseMethods.envUndefined = envUndefined;
    ;
    function Variable(parser, c) {
        TreeHelper_js_1.TreeHelper.printMethod("Variable");
        var def = {};
        if (parser.stack.env['font']) {
            def['mathvariant'] = parser.stack.env['font'];
        }
        var textNode = TreeHelper_js_1.TreeHelper.createText(c);
        var node = TreeHelper_js_1.TreeHelper.createNode('mi', [], def, textNode);
        parser.Push(parser.mmlToken(node));
    }
    ParseMethods.Variable = Variable;
    ;
    function Number(parser, c) {
        TreeHelper_js_1.TreeHelper.printMethod("Number");
        var mml;
        var n = parser.string.slice(parser.i - 1).match(NUMBER);
        var def = {};
        if (parser.stack.env['font']) {
            def['mathvariant'] = parser.stack.env['font'];
        }
        if (n) {
            var textNode = TreeHelper_js_1.TreeHelper.createText(n[0].replace(/[{}]/g, ""));
            mml = TreeHelper_js_1.TreeHelper.createNode('mn', [], def, textNode);
            parser.i += n[0].length - 1;
        }
        else {
            var textNode = TreeHelper_js_1.TreeHelper.createText(c);
            mml = TreeHelper_js_1.TreeHelper.createNode('mo', [], def, textNode);
        }
        parser.Push(parser.mmlToken(mml));
    }
    ParseMethods.Number = Number;
    ;
    function Open(parser, c) {
        parser.Push(new sitem.OpenItem());
    }
    ParseMethods.Open = Open;
    ;
    function Close(parser, c) {
        parser.Push(new sitem.CloseItem());
    }
    ParseMethods.Close = Close;
    ;
    function Tilde(parser, c) {
        var textNode = TreeHelper_js_1.TreeHelper.createText(NBSP);
        var node = TreeHelper_js_1.TreeHelper.createNode('mtext', [], {}, textNode);
        parser.Push(node);
    }
    ParseMethods.Tilde = Tilde;
    ;
    function Space(parser, c) { }
    ParseMethods.Space = Space;
    ;
    function Superscript(parser, c) {
        TreeHelper_js_1.TreeHelper.printMethod("Superscript");
        if (parser.GetNext().match(/\d/)) {
            parser.string = parser.string.substr(0, parser.i + 1) + " " + parser.string.substr(parser.i + 1);
        }
        var primes, base, top = parser.stack.Top();
        if (top.hasType('prime')) {
            base = top.data[0];
            primes = top.data[1];
            parser.stack.Pop();
        }
        else {
            base = parser.stack.Prev();
            if (!base) {
                var textNode = TreeHelper_js_1.TreeHelper.createText("");
                base = TreeHelper_js_1.TreeHelper.createNode('mi', [], {}, textNode);
            }
        }
        var movesupsub = TreeHelper_js_1.TreeHelper.getProperty(base, 'movesupsub');
        var position = TreeHelper_js_1.TreeHelper.isType(base, "msubsup") ? base.sup :
            base.over;
        if ((TreeHelper_js_1.TreeHelper.isType(base, "msubsup") && TreeHelper_js_1.TreeHelper.getChildAt(base, base.sup)) ||
            (TreeHelper_js_1.TreeHelper.isType(base, "munderover") && TreeHelper_js_1.TreeHelper.getChildAt(base, base.over) &&
                !TreeHelper_js_1.TreeHelper.getProperty(base, 'subsupOK'))) {
            throw new TexError_js_1.default(["DoubleExponent", "Double exponent: use braces to clarify"]);
        }
        if (!TreeHelper_js_1.TreeHelper.isType(base, "msubsup")) {
            TreeHelper_js_1.TreeHelper.printSimple('Case 1');
            if (movesupsub) {
                TreeHelper_js_1.TreeHelper.printSimple('Case 2');
                if (!TreeHelper_js_1.TreeHelper.isType(base, "munderover") || TreeHelper_js_1.TreeHelper.getChildAt(base, base.over)) {
                    TreeHelper_js_1.TreeHelper.printSimple('Case 3');
                    if (TreeHelper_js_1.TreeHelper.getProperty(base, 'movablelimits') && TreeHelper_js_1.TreeHelper.isType(base, 'mi')) {
                        base = ParseMethods.mi2mo(base);
                    }
                    base = TreeHelper_js_1.TreeHelper.createNode('munderover', [base], { movesupsub: true });
                }
                position = base.over;
            }
            else {
                TreeHelper_js_1.TreeHelper.printSimple('Case 4');
                base = TreeHelper_js_1.TreeHelper.createNode('msubsup', [base], {});
                position = base.sup;
            }
        }
        parser.Push(new sitem.SubsupItem(base).With({
            position: position, primes: primes, movesupsub: movesupsub
        }));
    }
    ParseMethods.Superscript = Superscript;
    ;
    function Subscript(parser, c) {
        TreeHelper_js_1.TreeHelper.printMethod("Subscript");
        if (parser.GetNext().match(/\d/)) {
            parser.string = parser.string.substr(0, parser.i + 1) + " " + parser.string.substr(parser.i + 1);
        }
        var primes, base, top = parser.stack.Top();
        if (top.hasType('prime')) {
            base = top.data[0];
            primes = top.data[1];
            parser.stack.Pop();
        }
        else {
            base = parser.stack.Prev();
            if (!base) {
                var textNode = TreeHelper_js_1.TreeHelper.createText("");
                base = TreeHelper_js_1.TreeHelper.createNode('mi', [], {}, textNode);
            }
        }
        var movesupsub = TreeHelper_js_1.TreeHelper.getProperty(base, 'movesupsub');
        var position = TreeHelper_js_1.TreeHelper.isType(base, "msubsup") ? base.sub : base.under;
        if ((TreeHelper_js_1.TreeHelper.isType(base, "msubsup") && TreeHelper_js_1.TreeHelper.getChildAt(base, base.sub)) ||
            (TreeHelper_js_1.TreeHelper.isType(base, "munderover") && TreeHelper_js_1.TreeHelper.getChildAt(base, base.under) &&
                !TreeHelper_js_1.TreeHelper.getProperty(base, 'subsupOK'))) {
            throw new TexError_js_1.default(["DoubleSubscripts", "Double subscripts: use braces to clarify"]);
        }
        if (!TreeHelper_js_1.TreeHelper.isType(base, "msubsup")) {
            if (movesupsub) {
                if (!TreeHelper_js_1.TreeHelper.isType(base, "munderover") || TreeHelper_js_1.TreeHelper.getChildAt(base, base.under)) {
                    if (TreeHelper_js_1.TreeHelper.getProperty(base, 'movablelimits') && TreeHelper_js_1.TreeHelper.isType(base, 'mi')) {
                        base = ParseMethods.mi2mo(base);
                    }
                    base = TreeHelper_js_1.TreeHelper.createNode('munderover', [base], { movesupsub: true });
                }
                position = base.under;
            }
            else {
                base = TreeHelper_js_1.TreeHelper.createNode('msubsup', [base], {});
                position = base.sub;
            }
        }
        parser.Push(new sitem.SubsupItem(base).With({
            position: position, primes: primes, movesupsub: movesupsub
        }));
    }
    ParseMethods.Subscript = Subscript;
    ;
    function Prime(parser, c) {
        var base = parser.stack.Prev();
        if (!base) {
            base = TreeHelper_js_1.TreeHelper.createNode('mi', [], {});
        }
        if (TreeHelper_js_1.TreeHelper.isType(base, "msubsup") && TreeHelper_js_1.TreeHelper.getChildAt(base, base.sup)) {
            throw new TexError_js_1.default(["DoubleExponentPrime",
                "Prime causes double exponent: use braces to clarify"]);
        }
        var sup = "";
        parser.i--;
        do {
            sup += PRIME;
            parser.i++, c = parser.GetNext();
        } while (c === "'" || c === SMARTQUOTE);
        sup = ["", "\u2032", "\u2033", "\u2034", "\u2057"][sup.length] || sup;
        var textNode = TreeHelper_js_1.TreeHelper.createText(sup);
        var node = TreeHelper_js_1.TreeHelper.createNode('mo', [], {}, textNode);
        parser.Push(new sitem.PrimeItem(base, parser.mmlToken(node)));
    }
    ParseMethods.Prime = Prime;
    ;
    function Comment(parser, c) {
        TreeHelper_js_1.TreeHelper.printMethod("Comment");
        while (parser.i < parser.string.length && parser.string.charAt(parser.i) != "\n") {
            parser.i++;
        }
    }
    ParseMethods.Comment = Comment;
    ;
    function Hash(parser, c) {
        TreeHelper_js_1.TreeHelper.printMethod("Hash");
        throw new TexError_js_1.default(["CantUseHash1",
            "You can't use 'macro parameter character #' in math mode"]);
    }
    ParseMethods.Hash = Hash;
    ;
    function Other(parser, c) {
        TreeHelper_js_1.TreeHelper.printMethod("Other");
        var def = {};
        if (parser.stack.env['font']) {
            def = { mathvariant: parser.stack.env['font'] };
        }
        var remap = parser.remap.lookup(c);
        var textNode = TreeHelper_js_1.TreeHelper.createText(remap ? remap.char : c);
        var mo = TreeHelper_js_1.TreeHelper.createNode('mo', [], def, textNode);
        parser.Push(parser.mmlToken(mo));
    }
    ParseMethods.Other = Other;
    ;
    function SetFont(parser, name, font) {
        parser.stack.env['font'] = font;
    }
    ParseMethods.SetFont = SetFont;
    ;
    function SetStyle(parser, name, texStyle, style, level) {
        TreeHelper_js_1.TreeHelper.printMethod("SetStyle: " + name + " texStyle: " + texStyle +
            " style: " + style + " level: " + level);
        parser.stack.env['style'] = texStyle;
        parser.stack.env['level'] = level;
        parser.Push(new sitem.StyleItem().With({ styles: { displaystyle: style, scriptlevel: level } }));
    }
    ParseMethods.SetStyle = SetStyle;
    ;
    function SetSize(parser, name, size) {
        TreeHelper_js_1.TreeHelper.printMethod("SetSize");
        parser.stack.env['size'] = size;
        parser.Push(new sitem.StyleItem().With({ styles: { mathsize: size + "em" } }));
    }
    ParseMethods.SetSize = SetSize;
    ;
    function Color(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("Color");
        var color = parser.GetArgument(name);
        var old = parser.stack.env['color'];
        parser.stack.env['color'] = color;
        var math = parser.ParseArg(name);
        if (old) {
            parser.stack.env['color'];
        }
        else {
            delete parser.stack.env['color'];
        }
        var node = TreeHelper_js_1.TreeHelper.createNode('mstyle', [math], { mathcolor: color });
        parser.Push(node);
    }
    ParseMethods.Color = Color;
    ;
    function Spacer(parser, name, space) {
        var node = TreeHelper_js_1.TreeHelper.createNode('mspace', [], { width: space, mathsize: TexConstants_js_1.TexConstant.Size.NORMAL, scriptlevel: 0 });
        parser.Push(node);
    }
    ParseMethods.Spacer = Spacer;
    ;
    function LeftRight(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("LeftRight");
        var alpha = name.substr(1);
        parser.Push(new sitem[alpha[0].toUpperCase() + alpha.slice(1) + 'Item']()
            .With({ delim: parser.GetDelimiter(name) }));
    }
    ParseMethods.LeftRight = LeftRight;
    ;
    function Middle(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("Middle");
        var delim = parser.GetDelimiter(name);
        var node = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [], { texClass: MmlNode_js_1.TEXCLASS.CLOSE });
        parser.Push(node);
        if (!parser.stack.Top().hasType('left')) {
            throw new TexError_js_1.default(["MisplacedMiddle", "%1 must be within \\left and \\right", name]);
        }
        var textNode = TreeHelper_js_1.TreeHelper.createText(delim);
        node = TreeHelper_js_1.TreeHelper.createNode('mo', [], { stretchy: true }, textNode);
        parser.Push(node);
        node = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [], { texClass: MmlNode_js_1.TEXCLASS.OPEN });
        parser.Push(node);
    }
    ParseMethods.Middle = Middle;
    ;
    function NamedFn(parser, name, id) {
        TreeHelper_js_1.TreeHelper.printMethod("NamedFn");
        if (!id) {
            id = name.substr(1);
        }
        ;
        var textNode = TreeHelper_js_1.TreeHelper.createText(id);
        var mml = TreeHelper_js_1.TreeHelper.createNode('mi', [], { texClass: MmlNode_js_1.TEXCLASS.OP }, textNode);
        parser.Push(new sitem.FnItem(parser.mmlToken(mml)));
    }
    ParseMethods.NamedFn = NamedFn;
    ;
    function NamedOp(parser, name, id) {
        TreeHelper_js_1.TreeHelper.printMethod("NamedOp");
        if (!id) {
            id = name.substr(1);
        }
        ;
        id = id.replace(/&thinsp;/, "\u2006");
        var text = TreeHelper_js_1.TreeHelper.createText(id);
        var mml = TreeHelper_js_1.TreeHelper.createNode('mo', [], {
            movablelimits: true,
            movesupsub: true,
            form: TexConstants_js_1.TexConstant.Form.PREFIX,
            texClass: MmlNode_js_1.TEXCLASS.OP
        }, text);
        parser.Push(parser.mmlToken(mml));
    }
    ParseMethods.NamedOp = NamedOp;
    ;
    function Limits(parser, name, limits) {
        TreeHelper_js_1.TreeHelper.printMethod("Limits");
        var op = parser.stack.Prev(true);
        if (!op || (TreeHelper_js_1.TreeHelper.getTexClass(op) !== MmlNode_js_1.TEXCLASS.OP &&
            TreeHelper_js_1.TreeHelper.getProperty(op, 'movesupsub') == null)) {
            throw new TexError_js_1.default(["MisplacedLimits", "%1 is allowed only on operators", name]);
        }
        var top = parser.stack.Top();
        if (TreeHelper_js_1.TreeHelper.isType(op, "munderover") && !limits) {
            var node = TreeHelper_js_1.TreeHelper.createNode('msubsup', [], {});
            TreeHelper_js_1.TreeHelper.copyChildren(op, node);
            op = top.data[top.data.length - 1] = node;
        }
        else if (TreeHelper_js_1.TreeHelper.isType(op, "msubsup") && limits) {
            node = TreeHelper_js_1.TreeHelper.createNode('munderover', [], {});
            TreeHelper_js_1.TreeHelper.copyChildren(op, node);
            op = top.data[top.data.length - 1] = node;
        }
        TreeHelper_js_1.TreeHelper.setProperties(op, { 'movesupsub': limits ? true : false });
        TreeHelper_js_1.TreeHelper.setProperties(TreeHelper_js_1.TreeHelper.getCore(op), { 'movablelimits': false });
        if (TreeHelper_js_1.TreeHelper.getProperty(op, 'movablelimits')) {
            TreeHelper_js_1.TreeHelper.setProperties(op, { 'movablelimits': false });
        }
    }
    ParseMethods.Limits = Limits;
    ;
    function Over(parser, name, open, close) {
        TreeHelper_js_1.TreeHelper.printMethod("Over");
        var mml = new sitem.OverItem().With({ name: name });
        if (open || close) {
            mml.setProperty('open', open);
            mml.setProperty('close', close);
        }
        else if (name.match(/withdelims$/)) {
            mml.setProperty('open', parser.GetDelimiter(name));
            mml.setProperty('close', parser.GetDelimiter(name));
        }
        if (name.match(/^\\above/)) {
            mml.setProperty('thickness', parser.GetDimen(name));
        }
        else if (name.match(/^\\atop/) || open || close) {
            mml.setProperty('thickness', 0);
        }
        parser.Push(mml);
    }
    ParseMethods.Over = Over;
    ;
    function Frac(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("Frac");
        var num = parser.ParseArg(name);
        var den = parser.ParseArg(name);
        var node = TreeHelper_js_1.TreeHelper.createNode('mfrac', [num, den], {});
        parser.Push(node);
    }
    ParseMethods.Frac = Frac;
    ;
    function Sqrt(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("Sqrt");
        var n = parser.GetBrackets(name), arg = parser.GetArgument(name);
        if (arg === "\\frac") {
            arg += "{" + parser.GetArgument(arg) + "}{" + parser.GetArgument(arg) + "}";
        }
        var mml = new OldParser_js_1.OldParser(arg, parser.stack.env, parser.ParseMethods).mml();
        if (!n) {
            mml = TreeHelper_js_1.TreeHelper.createNode('msqrt', [mml], {});
        }
        else {
            mml = TreeHelper_js_1.TreeHelper.createNode('mroot', [mml, ParseMethods.parseRoot(parser, n)], {});
        }
        parser.Push(mml);
    }
    ParseMethods.Sqrt = Sqrt;
    ;
    function Root(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("Root");
        var n = parser.GetUpTo(name, "\\of");
        var arg = parser.ParseArg(name);
        var node = TreeHelper_js_1.TreeHelper.createNode('mroot', [arg, ParseMethods.parseRoot(parser, n)], {});
        parser.Push(node);
    }
    ParseMethods.Root = Root;
    ;
    function parseRoot(parser, n) {
        TreeHelper_js_1.TreeHelper.printMethod("parseRoot");
        var env = parser.stack.env, inRoot = env['inRoot'];
        env['inRoot'] = true;
        var parser = new OldParser_js_1.OldParser(n, env, parser.ParseMethods);
        var node = parser.mml();
        TreeHelper_js_1.TreeHelper.printJSON(node);
        var global = parser.stack.global;
        if (global['leftRoot'] || global['upRoot']) {
            var def = {};
            if (global['leftRoot']) {
                def['width'] = global['leftRoot'];
            }
            if (global['upRoot']) {
                def['voffset'] = global['upRoot'];
                def['height'] = global['upRoot'];
            }
            node = TreeHelper_js_1.TreeHelper.createNode('mpadded', [node], def);
        }
        env['inRoot'] = inRoot;
        return node;
    }
    ParseMethods.parseRoot = parseRoot;
    ;
    function MoveRoot(parser, name, id) {
        TreeHelper_js_1.TreeHelper.printMethod("MoveRoot");
        if (!parser.stack.env['inRoot']) {
            throw new TexError_js_1.default(["MisplacedMoveRoot", "%1 can appear only within a root", name]);
        }
        if (parser.stack.global[id]) {
            throw new TexError_js_1.default(["MultipleMoveRoot", "Multiple use of %1", name]);
        }
        var n = parser.GetArgument(name);
        if (!n.match(/-?[0-9]+/)) {
            throw new TexError_js_1.default(["IntegerArg", "The argument to %1 must be an integer", name]);
        }
        n = (parseInt(n, 10) / 15) + "em";
        if (n.substr(0, 1) !== "-") {
            n = "+" + n;
        }
        parser.stack.global[id] = n;
    }
    ParseMethods.MoveRoot = MoveRoot;
    ;
    function Accent(parser, name, accent, stretchy) {
        TreeHelper_js_1.TreeHelper.printMethod("Accent");
        var c = parser.ParseArg(name);
        var def = { accent: true };
        if (parser.stack.env['font']) {
            def['mathvariant'] = parser.stack.env['font'];
        }
        var entity = TreeHelper_js_1.TreeHelper.createEntity(accent);
        var moNode = TreeHelper_js_1.TreeHelper.createNode('mo', [], def, entity);
        var mml = parser.mmlToken(moNode);
        TreeHelper_js_1.TreeHelper.setProperties(mml, { stretchy: (stretchy ? true : false) });
        var mo = (TreeHelper_js_1.TreeHelper.isEmbellished(c) ? TreeHelper_js_1.TreeHelper.getCoreMO(c) : c);
        if (TreeHelper_js_1.TreeHelper.isType(mo, 'mo')) {
            TreeHelper_js_1.TreeHelper.setProperties(mo, { 'movablelimits': false });
        }
        var muoNode = TreeHelper_js_1.TreeHelper.createNode('munderover', [], { accent: true });
        TreeHelper_js_1.TreeHelper.setData(muoNode, 0, c);
        TreeHelper_js_1.TreeHelper.setData(muoNode, 1, null);
        TreeHelper_js_1.TreeHelper.setData(muoNode, 2, mml);
        var texAtom = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [muoNode], {});
        parser.Push(texAtom);
    }
    ParseMethods.Accent = Accent;
    ;
    function UnderOver(parser, name, c, stack, noaccent) {
        TreeHelper_js_1.TreeHelper.printMethod("UnderOver");
        var base = parser.ParseArg(name);
        if (TreeHelper_js_1.TreeHelper.getProperty(base, 'movablelimits')) {
            TreeHelper_js_1.TreeHelper.setProperties(base, { 'movablelimits': false });
        }
        if (TreeHelper_js_1.TreeHelper.isType(base, 'munderover') && TreeHelper_js_1.TreeHelper.isEmbellished(base)) {
            TreeHelper_js_1.TreeHelper.setProperties(TreeHelper_js_1.TreeHelper.getCore(base), { lspace: 0, rspace: 0 });
            var mo = TreeHelper_js_1.TreeHelper.createNode('mo', [], { rspace: 0 });
            base = TreeHelper_js_1.TreeHelper.createNode('mrow', [mo, base], {});
        }
        var mml = TreeHelper_js_1.TreeHelper.createNode('munderover', [base], {});
        var entity = TreeHelper_js_1.TreeHelper.createEntity(c);
        mo = TreeHelper_js_1.TreeHelper.createNode('mo', [], { stretchy: true, accent: !noaccent }, entity);
        TreeHelper_js_1.TreeHelper.setData(mml, name.charAt(1) === "o" ? mml.over : mml.under, parser.mmlToken(mo));
        var node = mml;
        if (stack) {
            TreeHelper_js_1.TreeHelper.untested(8);
            node = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [mml], { texClass: MmlNode_js_1.TEXCLASS.OP, movesupsub: true });
        }
        TreeHelper_js_1.TreeHelper.setProperties(node, { subsupOK: true });
        parser.Push(node);
    }
    ParseMethods.UnderOver = UnderOver;
    ;
    function Overset(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("Overset");
        var top = parser.ParseArg(name), base = parser.ParseArg(name);
        if (TreeHelper_js_1.TreeHelper.getProperty(base, 'movablelimits')) {
            TreeHelper_js_1.TreeHelper.setProperties(base, { 'movablelimits': false });
        }
        var node = TreeHelper_js_1.TreeHelper.createNode('mover', [base, top], {});
        parser.Push(node);
    }
    ParseMethods.Overset = Overset;
    ;
    function Underset(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("Underset");
        var bot = parser.ParseArg(name), base = parser.ParseArg(name);
        if (TreeHelper_js_1.TreeHelper.getProperty(base, 'movablelimits')) {
            TreeHelper_js_1.TreeHelper.setProperties(base, { 'movablelimits': false });
        }
        var node = TreeHelper_js_1.TreeHelper.createNode('munder', [base, bot], {});
        parser.Push(node);
    }
    ParseMethods.Underset = Underset;
    ;
    function TeXAtom(parser, name, mclass) {
        TreeHelper_js_1.TreeHelper.printMethod("TeXAtom");
        var def = { texClass: mclass };
        var mml;
        var node;
        if (mclass == MmlNode_js_1.TEXCLASS.OP) {
            def['movesupsub'] = def['movablelimits'] = true;
            var arg = parser.GetArgument(name);
            var match = arg.match(/^\s*\\rm\s+([a-zA-Z0-9 ]+)$/);
            if (match) {
                def['mathvariant'] = TexConstants_js_1.TexConstant.Variant.NORMAL;
                var textNode = TreeHelper_js_1.TreeHelper.createText(match[1]);
                node = TreeHelper_js_1.TreeHelper.createNode('mi', [], def, textNode);
                mml = new sitem.FnItem(parser.mmlToken(node));
            }
            else {
                var parsed = new OldParser_js_1.OldParser(arg, parser.stack.env, parser.ParseMethods).mml();
                node = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [parsed], def);
                mml = new sitem.FnItem(node);
            }
        }
        else {
            parsed = parser.ParseArg(name);
            mml = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [parsed], def);
        }
        parser.Push(mml);
    }
    ParseMethods.TeXAtom = TeXAtom;
    ;
    function MmlToken(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("MmlToken");
        var type = parser.GetArgument(name), attr = parser.GetBrackets(name, "").replace(/^\s+/, ""), data = parser.GetArgument(name), def = {};
        var node;
        try {
            node = TreeHelper_js_1.TreeHelper.createNode(type, [], {});
        }
        catch (e) {
            node = null;
        }
        if (!node || !node.isToken) {
            throw new TexError_js_1.default(["NotMathMLToken", "%1 is not a token element", type]);
        }
        while (attr !== "") {
            var match = attr.match(/^([a-z]+)\s*=\s*('[^']*'|"[^"]*"|[^ ,]*)\s*,?\s*/i);
            if (!match) {
                throw new TexError_js_1.default(["InvalidMathMLAttr", "Invalid MathML attribute: %1", attr]);
            }
            if (node.attributes.getAllDefaults()[match[1]] == null && !MmlTokenAllow[match[1]]) {
                throw new TexError_js_1.default(["UnknownAttrForElement",
                    "%1 is not a recognized attribute for %2",
                    match[1], type]);
            }
            var value = ParseMethods.MmlFilterAttribute(parser, match[1], match[2].replace(/^(['"])(.*)\1$/, "$2"));
            if (value) {
                if (value.toLowerCase() === "true") {
                    value = true;
                }
                else if (value.toLowerCase() === "false") {
                    value = false;
                }
                def[match[1]] = value;
            }
            attr = attr.substr(match[0].length);
        }
        TreeHelper_js_1.TreeHelper.printSimple("End mmlToken: type: " + type + " data: " + data + ' def: ');
        var textNode = TreeHelper_js_1.TreeHelper.createText(data);
        node.appendChild(textNode);
        TreeHelper_js_1.TreeHelper.setProperties(node, def);
        parser.Push(parser.mmlToken(node));
    }
    ParseMethods.MmlToken = MmlToken;
    ;
    function Strut(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("Strut");
        var row = TreeHelper_js_1.TreeHelper.createNode('mrow', [], {});
        var padded = TreeHelper_js_1.TreeHelper.createNode('mpadded', [row], { height: "8.6pt", depth: "3pt", width: 0 });
        parser.Push(padded);
    }
    ParseMethods.Strut = Strut;
    ;
    function Phantom(parser, name, v, h) {
        TreeHelper_js_1.TreeHelper.printMethod("Phantom");
        var box = TreeHelper_js_1.TreeHelper.createNode('mphantom', [parser.ParseArg(name)], {});
        if (v || h) {
            box = TreeHelper_js_1.TreeHelper.createNode('mpadded', [box], {});
            if (h) {
                TreeHelper_js_1.TreeHelper.setAttribute(box, 'height', 0);
                TreeHelper_js_1.TreeHelper.setAttribute(box, 'depth', 0);
            }
            if (v) {
                TreeHelper_js_1.TreeHelper.setAttribute(box, 'width', 0);
            }
        }
        var atom = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [box], {});
        parser.Push(atom);
    }
    ParseMethods.Phantom = Phantom;
    ;
    function Smash(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("Smash");
        var bt = parser.trimSpaces(parser.GetBrackets(name, ""));
        var smash = TreeHelper_js_1.TreeHelper.createNode('mpadded', [parser.ParseArg(name)], {});
        switch (bt) {
            case "b":
                TreeHelper_js_1.TreeHelper.setAttribute(smash, 'depth', 0);
                break;
            case "t":
                TreeHelper_js_1.TreeHelper.setAttribute(smash, 'height', 0);
                break;
            default:
                TreeHelper_js_1.TreeHelper.setAttribute(smash, 'height', 0);
                TreeHelper_js_1.TreeHelper.setAttribute(smash, 'depth', 0);
        }
        var atom = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [smash], {});
        parser.Push(atom);
    }
    ParseMethods.Smash = Smash;
    ;
    function Lap(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("Lap");
        var mml = TreeHelper_js_1.TreeHelper.createNode('mpadded', [parser.ParseArg(name)], { width: 0 });
        if (name === "\\llap") {
            TreeHelper_js_1.TreeHelper.setAttribute(mml, 'lspace', '-1width');
        }
        var atom = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [mml], {});
        parser.Push(atom);
    }
    ParseMethods.Lap = Lap;
    ;
    function RaiseLower(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("RaiseLower");
        var h = parser.GetDimen(name);
        var item = new sitem.PositionItem().With({ name: name, move: 'vertical' });
        if (h.charAt(0) === '-') {
            h = h.slice(1);
            name = name.substr(1) === 'raise' ? '\\lower' : '\\raise';
        }
        if (name === '\\lower') {
            item.setProperty('dh', '-' + h);
            item.setProperty('dd', '+' + h);
        }
        else {
            item.setProperty('dh', '+' + h);
            item.setProperty('dd', '-' + h);
        }
        parser.Push(item);
    }
    ParseMethods.RaiseLower = RaiseLower;
    ;
    function MoveLeftRight(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("MoveLeftRight");
        var h = parser.GetDimen(name);
        var nh = (h.charAt(0) === '-' ? h.slice(1) : '-' + h);
        if (name === "\\moveleft") {
            var tmp = h;
            h = nh;
            nh = tmp;
        }
        parser.Push(new sitem.PositionItem().With({
            name: name, move: 'horizontal',
            left: TreeHelper_js_1.TreeHelper.createNode('mspace', [], { width: h, mathsize: TexConstants_js_1.TexConstant.Size.NORMAL }),
            right: TreeHelper_js_1.TreeHelper.createNode('mspace', [], { width: nh, mathsize: TexConstants_js_1.TexConstant.Size.NORMAL })
        }));
    }
    ParseMethods.MoveLeftRight = MoveLeftRight;
    ;
    function Hskip(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("Hskip");
        var node = TreeHelper_js_1.TreeHelper.createNode('mspace', [], { width: parser.GetDimen(name),
            mathsize: TexConstants_js_1.TexConstant.Size.NORMAL });
        parser.Push(node);
    }
    ParseMethods.Hskip = Hskip;
    ;
    function Rule(parser, name, style) {
        TreeHelper_js_1.TreeHelper.printMethod("Rule");
        var w = parser.GetDimen(name), h = parser.GetDimen(name), d = parser.GetDimen(name);
        var def = { width: w, height: h, depth: d };
        if (style !== 'blank') {
            def['mathbackground'] = (parser.stack.env['color'] || "black");
        }
        var node = TreeHelper_js_1.TreeHelper.createNode('mspace', [], def);
        parser.Push(node);
    }
    ParseMethods.Rule = Rule;
    ;
    function rule(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("rule");
        var v = parser.GetBrackets(name), w = parser.GetDimen(name), h = parser.GetDimen(name);
        var mml = TreeHelper_js_1.TreeHelper.createNode('mspace', [], {
            width: w, height: h,
            mathbackground: (parser.stack.env['color'] || "black")
        });
        if (v) {
            mml = TreeHelper_js_1.TreeHelper.createNode('mpadded', [mml], { voffset: v });
            if (v.match(/^\-/)) {
                TreeHelper_js_1.TreeHelper.setAttribute(mml, 'height', v);
                TreeHelper_js_1.TreeHelper.setAttribute(mml, 'depth', '+' + v.substr(1));
            }
            else {
                TreeHelper_js_1.TreeHelper.setAttribute(mml, 'height', '+' + v);
            }
        }
        parser.Push(mml);
    }
    ParseMethods.rule = rule;
    ;
    function MakeBig(parser, name, mclass, size) {
        TreeHelper_js_1.TreeHelper.printMethod("MakeBig");
        size *= P_HEIGHT;
        var sizeStr = String(size).replace(/(\.\d\d\d).+/, '$1') + "em";
        var delim = parser.GetDelimiter(name, true);
        var text = TreeHelper_js_1.TreeHelper.createText(delim);
        var mo = TreeHelper_js_1.TreeHelper.createNode('mo', [], {
            minsize: sizeStr, maxsize: sizeStr,
            fence: true, stretchy: true, symmetric: true
        }, text);
        var node = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [mo], { texClass: mclass });
        parser.Push(node);
    }
    ParseMethods.MakeBig = MakeBig;
    ;
    function BuildRel(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("BuildRel");
        var top = parser.ParseUpTo(name, "\\over");
        var bot = parser.ParseArg(name);
        var node = TreeHelper_js_1.TreeHelper.createNode('munderover', [], {});
        TreeHelper_js_1.TreeHelper.setData(node, 0, bot);
        TreeHelper_js_1.TreeHelper.setData(node, 1, null);
        TreeHelper_js_1.TreeHelper.setData(node, 2, top);
        var atom = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [node], { texClass: MmlNode_js_1.TEXCLASS.REL });
        parser.Push(atom);
    }
    ParseMethods.BuildRel = BuildRel;
    ;
    function HBox(parser, name, style) {
        TreeHelper_js_1.TreeHelper.printMethod("HBox");
        parser.PushAll(parser.InternalMath(parser.GetArgument(name), style));
    }
    ParseMethods.HBox = HBox;
    ;
    function FBox(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("FBox");
        var internal = parser.InternalMath(parser.GetArgument(name));
        var node = TreeHelper_js_1.TreeHelper.createNode('menclose', internal, { notation: "box" });
        parser.Push(node);
    }
    ParseMethods.FBox = FBox;
    ;
    function Not(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("Not");
        parser.Push(new sitem.NotItem());
    }
    ParseMethods.Not = Not;
    ;
    function Dots(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("Dots");
        var ldotsEntity = TreeHelper_js_1.TreeHelper.createEntity('2026');
        var cdotsEntity = TreeHelper_js_1.TreeHelper.createEntity('22EF');
        var ldots = TreeHelper_js_1.TreeHelper.createNode('mo', [], { stretchy: false }, ldotsEntity);
        var cdots = TreeHelper_js_1.TreeHelper.createNode('mo', [], { stretchy: false }, cdotsEntity);
        parser.Push(new sitem.DotsItem().With({
            ldots: parser.mmlToken(ldots),
            cdots: parser.mmlToken(cdots)
        }));
    }
    ParseMethods.Dots = Dots;
    ;
    function Matrix(parser, name, open, close, align, spacing, vspacing, style, cases, numbered) {
        TreeHelper_js_1.TreeHelper.printMethod("Matrix");
        var c = parser.GetNext();
        if (c === "") {
            throw new TexError_js_1.default(["MissingArgFor", "Missing argument for %1", name]);
        }
        if (c === "{") {
            parser.i++;
        }
        else {
            parser.string = c + "}" + parser.string.slice(parser.i + 1);
            parser.i = 0;
        }
        var array = new sitem.ArrayItem().With({ requireClose: true });
        array.arraydef = {
            rowspacing: (vspacing || "4pt"),
            columnspacing: (spacing || "1em")
        };
        if (cases) {
            array.setProperty('isCases', true);
        }
        if (numbered) {
            array.setProperty('isNumbered', true);
            array.arraydef.side = numbered;
        }
        if (open || close) {
            array.setProperty('open', open);
            array.setProperty('close', close);
        }
        if (style === "D") {
            array.arraydef.displaystyle = true;
        }
        if (align != null) {
            array.arraydef.columnalign = align;
        }
        parser.Push(array);
    }
    ParseMethods.Matrix = Matrix;
    ;
    function Entry(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("Entry");
        parser.Push(new sitem.CellItem().With({ isEntry: true, name: name }));
        if (parser.stack.Top().getProperty('isCases')) {
            var string = parser.string;
            var braces = 0, close = -1, i = parser.i, m = string.length;
            while (i < m) {
                var c = string.charAt(i);
                if (c === "{") {
                    braces++;
                    i++;
                }
                else if (c === "}") {
                    if (braces === 0) {
                        m = 0;
                    }
                    else {
                        braces--;
                        if (braces === 0 && close < 0) {
                            close = i - parser.i;
                        }
                        i++;
                    }
                }
                else if (c === "&" && braces === 0) {
                    throw new TexError_js_1.default(["ExtraAlignTab", "Extra alignment tab in \\cases text"]);
                }
                else if (c === "\\") {
                    if (string.substr(i).match(/^((\\cr)[^a-zA-Z]|\\\\)/)) {
                        m = 0;
                    }
                    else {
                        i += 2;
                    }
                }
                else {
                    i++;
                }
            }
            var text = string.substr(parser.i, i - parser.i);
            if (!text.match(/^\s*\\text[^a-zA-Z]/) || close !== text.replace(/\s+$/, '').length - 1) {
                var internal = parser.InternalMath(text, 0);
                parser.PushAll(internal);
                parser.i = i;
            }
        }
    }
    ParseMethods.Entry = Entry;
    ;
    function Cr(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("Cr");
        TreeHelper_js_1.TreeHelper.untested(15);
        parser.Push(new sitem.CellItem().With({ isCR: true, name: name }));
    }
    ParseMethods.Cr = Cr;
    ;
    function CrLaTeX(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("CrLaTeX");
        var n;
        if (parser.string.charAt(parser.i) === "[") {
            n = parser.GetBrackets(name, "").replace(/ /g, "").replace(/,/, ".");
            if (n && !ParserUtil_js_1.ParserUtil.matchDimen(n)) {
                throw new TexError_js_1.default(["BracketMustBeDimension",
                    "Bracket argument to %1 must be a dimension", name]);
            }
        }
        parser.Push(new sitem.CellItem().With({ isCR: true, name: name, linebreak: true }));
        var top = parser.stack.Top();
        if (top instanceof sitem.ArrayItem) {
            if (n && top.arraydef['rowspacing']) {
                var rows = top.arraydef['rowspacing'].split(/ /);
                if (!top.getProperty('rowspacing')) {
                    top.setProperty('rowspacing', ParserUtil_js_1.ParserUtil.dimen2em(rows[0]));
                }
                while (rows.length < top.table.length) {
                    rows.push(ParserUtil_js_1.ParserUtil.Em(top.getProperty('rowspacing')));
                }
                rows[top.table.length - 1] = ParserUtil_js_1.ParserUtil.Em(Math.max(0, top.getProperty('rowspacing') + ParserUtil_js_1.ParserUtil.dimen2em(n)));
                top.arraydef['rowspacing'] = rows.join(' ');
            }
        }
        else {
            if (n) {
                var node = TreeHelper_js_1.TreeHelper.createNode('mspace', [], { depth: n });
                parser.Push(node);
            }
            node = TreeHelper_js_1.TreeHelper.createNode('mspace', [], { linebreak: TexConstants_js_1.TexConstant.LineBreak.NEWLINE });
            parser.Push(node);
        }
    }
    ParseMethods.CrLaTeX = CrLaTeX;
    ;
    function HLine(parser, name, style) {
        TreeHelper_js_1.TreeHelper.printMethod("HLine");
        if (style == null) {
            style = "solid";
        }
        var top = parser.stack.Top();
        if (!(top instanceof sitem.ArrayItem) || top.data.length) {
            throw new TexError_js_1.default(["Misplaced", "Misplaced %1", name]);
        }
        if (top.table.length == 0) {
            top.frame.push("top");
        }
        else {
            var lines = (top.arraydef['rowlines'] ? top.arraydef['rowlines'].split(/ /) : []);
            while (lines.length < top.table.length) {
                lines.push("none");
            }
            lines[top.table.length - 1] = style;
            top.arraydef['rowlines'] = lines.join(' ');
        }
    }
    ParseMethods.HLine = HLine;
    ;
    function HFill(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("HFill");
        var top = parser.stack.Top();
        if (top instanceof sitem.ArrayItem) {
            top.hfill.push(top.data.length);
        }
        else {
            throw new TexError_js_1.default(["UnsupportedHFill", "Unsupported use of %1", name]);
        }
    }
    ParseMethods.HFill = HFill;
    ;
    function mi2mo(mi) {
        TreeHelper_js_1.TreeHelper.printMethod("mi2mo");
        var mo = TreeHelper_js_1.TreeHelper.createNode('mo', [], {});
        TreeHelper_js_1.TreeHelper.copyChildren(mi, mo);
        TreeHelper_js_1.TreeHelper.copyAttributes(mi, mo);
        TreeHelper_js_1.TreeHelper.setProperties(mo, { lspace: '0', rspace: '0' });
        return mo;
    }
    ParseMethods.mi2mo = mi2mo;
    ;
    var MAXMACROS = 10000;
    var MAXBUFFER = 5 * 1024;
    function BeginEnd(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("BeginEnd");
        var env = parser.GetArgument(name);
        var regexp = /^\\end\\/;
        if (env.match(regexp)) {
            env = env.substr(5);
        }
        if (env.match(/\\/i)) {
            throw new TexError_js_1.default(["InvalidEnv", "Invalid environment name '%1'", env]);
        }
        if (name === "\\end") {
            var mml = new sitem.EndItem().With({ name: env });
            parser.Push(mml);
        }
        else {
            if (++parser.macroCount > MAXMACROS) {
                throw new TexError_js_1.default(["MaxMacroSub2",
                    "MathJax maximum substitution count exceeded; " +
                        "is there a recursive latex environment?"]);
            }
            ParseMethods.NEW_PARSER.parse('environment', [env, this]);
        }
    }
    ParseMethods.BeginEnd = BeginEnd;
    ;
    function BeginEnvironment(parser, func, env, args) {
        TreeHelper_js_1.TreeHelper.printMethod("BeginEnvironment");
        var end = args[0];
        var mml = new sitem.BeginItem().With({ name: env, end: end });
        mml = func.apply(this, [parser, mml].concat(args.slice(1)));
        parser.Push(mml);
    }
    ParseMethods.BeginEnvironment = BeginEnvironment;
    ;
    function Equation(parser, begin, row) {
        return row;
    }
    ParseMethods.Equation = Equation;
    ;
    function Array(parser, begin, open, close, align, spacing, vspacing, style, raggedHeight) {
        TreeHelper_js_1.TreeHelper.printMethod("Array");
        if (!align) {
            align = parser.GetArgument("\\begin{" + begin.getName() + "}");
        }
        var lines = ("c" + align).replace(/[^clr|:]/g, '').replace(/[^|:]([|:])+/g, '$1');
        align = align.replace(/[^clr]/g, '').split('').join(' ');
        align = align.replace(/l/g, 'left').replace(/r/g, 'right').replace(/c/g, 'center');
        var array = new sitem.ArrayItem();
        array.arraydef = {
            columnalign: align,
            columnspacing: (spacing || "1em"),
            rowspacing: (vspacing || "4pt")
        };
        if (lines.match(/[|:]/)) {
            if (lines.charAt(0).match(/[|:]/)) {
                array.frame.push("left");
                array.dashed = lines.charAt(0) === ":";
            }
            if (lines.charAt(lines.length - 1).match(/[|:]/)) {
                array.frame.push("right");
            }
            lines = lines.substr(1, lines.length - 2);
            array.arraydef.columnlines =
                lines.split('').join(' ').replace(/[^|: ]/g, 'none').replace(/\|/g, 'solid').replace(/:/g, 'dashed');
        }
        if (open) {
            array.setProperty('open', parser.convertDelimiter(open));
        }
        if (close) {
            array.setProperty('close', parser.convertDelimiter(close));
        }
        if (style === "D") {
            array.arraydef.displaystyle = true;
        }
        else if (style) {
            array.arraydef.displaystyle = false;
        }
        if (style === "S") {
            array.arraydef.scriptlevel = 1;
        }
        if (raggedHeight) {
            array.arraydef.useHeight = false;
        }
        parser.Push(begin);
        return array;
    }
    ParseMethods.Array = Array;
    ;
    function AlignedArray(parser, begin) {
        TreeHelper_js_1.TreeHelper.printMethod("AlignedArray");
        var align = parser.GetBrackets("\\begin{" + begin.getName() + "}");
        return ParseMethods.setArrayAlign(parser, ParseMethods.Array.apply(parser, arguments), align);
    }
    ParseMethods.AlignedArray = AlignedArray;
    ;
    function setArrayAlign(parser, array, align) {
        TreeHelper_js_1.TreeHelper.printMethod("setArrayAlign");
        align = parser.trimSpaces(align || "");
        if (align === "t") {
            array.arraydef.align = "baseline 1";
        }
        else if (align === "b") {
            array.arraydef.align = "baseline -1";
        }
        else if (align === "c") {
            array.arraydef.align = "center";
        }
        else if (align) {
            array.arraydef.align = align;
        }
        return array;
    }
    ParseMethods.setArrayAlign = setArrayAlign;
    ;
    var EXTENSION_DIR = "";
    function Require(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod("Require");
        var file = parser.GetArgument(name)
            .replace(/.*\//, "")
            .replace(/[^a-z0-9_.-]/ig, "");
        ParseMethods.Extension(parser, null, file);
    }
    ParseMethods.Require = Require;
    ;
    function Extension(parser, name, file, array) {
        TreeHelper_js_1.TreeHelper.printMethod("Extension");
        if (name && !(typeof (name) === "string")) {
            name = name.getName();
        }
        file = EXTENSION_DIR + "/" + file;
        if (!file.match(/\.js$/)) {
            file += ".js";
        }
    }
    ParseMethods.Extension = Extension;
    ;
    function Macro(parser, name, macro, argcount, def) {
        TreeHelper_js_1.TreeHelper.printMethod("Macro");
        if (argcount) {
            var args = [];
            if (def != null) {
                var optional = parser.GetBrackets(name);
                args.push(optional == null ? def : optional);
            }
            for (var i = args.length; i < argcount; i++) {
                args.push(parser.GetArgument(name));
            }
            macro = ParseMethods.SubstituteArgs(args, macro);
        }
        parser.string = ParseMethods.AddArgs(macro, parser.string.slice(parser.i));
        parser.i = 0;
        if (++parser.macroCount > MAXMACROS) {
            throw new TexError_js_1.default(["MaxMacroSub1",
                "MathJax maximum macro substitution count exceeded; " +
                    "is there a recursive macro call?"]);
        }
    }
    ParseMethods.Macro = Macro;
    ;
    function SubstituteArgs(args, string) {
        TreeHelper_js_1.TreeHelper.printMethod("SubstituteArgs");
        var text = '';
        var newstring = '';
        var i = 0;
        while (i < string.length) {
            var c = string.charAt(i++);
            if (c === "\\") {
                text += c + string.charAt(i++);
            }
            else if (c === '#') {
                c = string.charAt(i++);
                if (c === '#') {
                    text += c;
                }
                else {
                    if (!c.match(/[1-9]/) || parseInt(c, 10) > args.length) {
                        throw new TexError_js_1.default(["IllegalMacroParam",
                            "Illegal macro parameter reference"]);
                    }
                    newstring = ParseMethods.AddArgs(this.AddArgs(newstring, text), args[parseInt(c, 10) - 1]);
                    text = '';
                }
            }
            else {
                text += c;
            }
        }
        return this.AddArgs(newstring, text);
    }
    ParseMethods.SubstituteArgs = SubstituteArgs;
    ;
    function AddArgs(s1, s2) {
        TreeHelper_js_1.TreeHelper.printMethod("AddArgs");
        if (s2.match(/^[a-z]/i) && s1.match(/(^|[^\\])(\\\\)*\\[a-z]+$/i)) {
            s1 += ' ';
        }
        if (s1.length + s2.length > MAXBUFFER) {
            throw new TexError_js_1.default(["MaxBufferSize",
                "MathJax internal buffer size exceeded; is there a recursive macro call?"]);
        }
        return s1 + s2;
    }
    ParseMethods.AddArgs = AddArgs;
    ;
    var TAG_SIDE = 'right';
    var TAG_INDENT = '0.8em';
    function AMSarray(parser, begin, numbered, taggable, align, spacing) {
        TreeHelper_js_1.TreeHelper.printMethod('AMS-AMSarray');
        parser.Push(begin);
        if (taggable) {
            ParseMethods.checkEqnEnv(parser);
        }
        align = align.replace(/[^clr]/g, '').split('').join(' ');
        align = align.replace(/l/g, 'left').replace(/r/g, 'right').replace(/c/g, 'center');
        var newItem = new sitem.AMSarrayItem(begin.getName(), numbered, taggable, parser.stack.global);
        newItem.arraydef = {
            displaystyle: true,
            rowspacing: ".5em",
            columnalign: align,
            columnspacing: (spacing || "1em"),
            side: TAG_SIDE,
            minlabelspacing: TAG_INDENT
        };
        return newItem;
    }
    ParseMethods.AMSarray = AMSarray;
    ;
    function checkEqnEnv(parser) {
        TreeHelper_js_1.TreeHelper.printMethod('AMS-checkEqnEnv');
        if (parser.stack.global.eqnenv) {
            throw new TexError_js_1.default(["ErroneousNestingEq", "Erroneous nesting of equation structures"]);
        }
        parser.stack.global.eqnenv = true;
    }
    ParseMethods.checkEqnEnv = checkEqnEnv;
    ;
    function HandleOperatorName(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod('AMS-HandleOperatorName');
        var limits = (parser.GetStar() ? "" : "\\nolimits\\SkipLimits");
        var op = parser.trimSpaces(parser.GetArgument(name));
        op = op.replace(/\*/g, '\\text{*}').replace(/-/g, '\\text{-}');
        parser.string = '\\mathop{\\rm ' + op + '}' + limits + " " + parser.string.slice(parser.i);
        parser.i = 0;
    }
    ParseMethods.HandleOperatorName = HandleOperatorName;
    ;
    function SkipLimits(parser, name) {
        TreeHelper_js_1.TreeHelper.printMethod('AMS-SkipLimits');
        var c = parser.GetNext(), i = parser.i;
        if (c === "\\" && ++parser.i && parser.GetCS() !== "limits")
            parser.i = i;
    }
    ParseMethods.SkipLimits = SkipLimits;
    ;
    function MathChoice(parser, name) {
        var D = parser.ParseArg(name);
        var T = parser.ParseArg(name);
        var S = parser.ParseArg(name);
        var SS = parser.ParseArg(name);
        parser.Push(Translate_js_1.NewTex.display ? D : T);
    }
    ParseMethods.MathChoice = MathChoice;
})(ParseMethods = exports.ParseMethods || (exports.ParseMethods = {}));
//# sourceMappingURL=ParseMethods.js.map