import { MmlNode } from '../../core/MmlTree/MmlNode.js';
export declare type EnvProp = string | number | boolean;
export declare type EnvList = {
    [key: string]: EnvProp;
};
export declare type Prop = string | number | boolean | MmlNode | PropList;
export declare type PropList = {
    [key: string]: Prop;
};
export declare type ErrorMsg = string[];
export declare type ErrorList = {
    [key: string]: ErrorMsg;
};
export declare type CheckType = boolean | StackItem | (MmlNode | StackItem)[];
export interface StackItem {
    checkItem(item: StackItem): CheckType;
    mmlData(inferred?: boolean, forceRow?: boolean): MmlNode;
    Pop(): MmlNode | void;
    Push(...args: MmlNode[]): void;
    hasType(kind: string): boolean;
    getType(): string;
    getProperty(key: string): Prop;
    setProperty(key: string, value: Prop): void;
    isClose: boolean;
    isOpen: boolean;
    data: MmlNode[];
    global: EnvList;
    getName(): string;
    env: EnvList;
}
export declare class BaseItem implements StackItem {
    private _env;
    private _properties;
    protected errors: {
        [key: string]: string[];
    };
    protected kind: string;
    global: EnvList;
    data: MmlNode[];
    constructor(...args: MmlNode[]);
    env: EnvList;
    getProperty(key: string): any;
    setProperty(key: string, value: Prop): void;
    readonly isOpen: boolean;
    readonly isClose: boolean;
    Push(...args: MmlNode[]): void;
    Pop(): MmlNode | void;
    getType(): string;
    hasType(kind: string): boolean;
    mmlData(inferred?: boolean, forceRow?: boolean): MmlNode;
    checkItem(item: StackItem): CheckType;
    With(def: PropList): this;
    toString(): string;
    getName(): string;
}
export declare class StartItem extends BaseItem {
    constructor(global: EnvList);
    readonly isOpen: boolean;
    checkItem(item: StackItem): boolean | StackItem | (MmlNode | StackItem)[] | MmlItem;
}
export declare class StopItem extends BaseItem {
    constructor();
    readonly isClose: boolean;
}
export declare class OpenItem extends BaseItem {
    constructor();
    readonly isOpen: boolean;
    checkItem(item: StackItem): boolean | StackItem | (MmlNode | StackItem)[] | MmlItem;
}
export declare class CloseItem extends BaseItem {
    constructor();
    readonly isClose: boolean;
}
export declare class PrimeItem extends BaseItem {
    constructor(...args: MmlNode[]);
    checkItem(item: StackItem): (MmlNode | StackItem)[];
}
export declare class SubsupItem extends BaseItem {
    constructor(...args: MmlNode[]);
    checkItem(item: StackItem): boolean | StackItem | (MmlNode | StackItem)[] | MmlItem;
    Pop(): void;
}
export declare class OverItem extends BaseItem {
    constructor();
    readonly isClose: boolean;
    checkItem(item: StackItem): boolean | StackItem | (MmlNode | StackItem)[] | (StackItem | MmlItem)[];
    toString(): string;
}
export declare class LeftItem extends BaseItem {
    constructor();
    readonly isOpen: boolean;
    checkItem(item: StackItem): boolean | StackItem | (MmlNode | StackItem)[] | MmlItem;
}
export declare class RightItem extends BaseItem {
    constructor();
    readonly isClose: boolean;
}
export declare class BeginItem extends BaseItem {
    constructor();
    readonly isOpen: boolean;
    checkItem(item: StackItem): boolean | StackItem | (MmlNode | StackItem)[] | MmlItem;
}
export declare class EndItem extends BaseItem {
    constructor();
    readonly isClose: boolean;
}
export declare class StyleItem extends BaseItem {
    constructor();
    checkItem(item: StackItem): boolean | StackItem | (MmlNode | StackItem)[] | (StackItem | MmlItem)[];
}
export declare class PositionItem extends BaseItem {
    constructor();
    checkItem(item: StackItem): boolean | StackItem | (MmlNode | StackItem)[] | (StackItem | MmlItem)[];
}
export declare class ArrayItem extends BaseItem {
    table: MmlNode[];
    row: MmlNode[];
    frame: string[];
    hfill: number[];
    copyEnv: boolean;
    arraydef: {
        [key: string]: string | number | boolean;
    };
    dashed: boolean;
    constructor();
    readonly isOpen: boolean;
    checkItem(item: StackItem): boolean | StackItem | (MmlNode | StackItem)[] | MmlItem | (StackItem | MmlItem)[];
    EndEntry(): void;
    EndRow(): void;
    EndTable(): void;
    checkLines(): void;
    clearEnv(): void;
}
export declare class CellItem extends BaseItem {
    constructor();
    readonly isClose: boolean;
}
export declare class MmlItem extends BaseItem {
    constructor(...args: MmlNode[]);
    Add(): this;
}
export declare class FnItem extends BaseItem {
    constructor(...args: MmlNode[]);
    checkItem(item: StackItem): any;
}
export declare class NotItem extends BaseItem {
    private remap;
    constructor();
    checkItem(item: StackItem): CheckType;
}
export declare class DotsItem extends BaseItem {
    constructor();
    checkItem(item: StackItem): CheckType;
}
export declare class AMSarrayItem extends ArrayItem {
    private numbered;
    private save;
    constructor(name: string, numbered: boolean, taggable: boolean, global: EnvList);
    autoTag(): void;
    getTag(): MmlNode;
    clearTag(): void;
    EndEntry(): void;
    EndRow(): void;
    EndTable(): void;
}
