"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var MapHandler_js_1 = require("./MapHandler.js");
var MmlEntities_js_1 = require("../mathml/MmlEntities.js");
var MmlNode_js_1 = require("../../core/MmlTree/MmlNode.js");
var TexError_js_1 = require("./TexError.js");
var ParserUtil_js_1 = require("./ParserUtil.js");
var TreeHelper_js_1 = require("./TreeHelper.js");
var BaseItem = (function () {
    function BaseItem() {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        this._properties = {};
        this.errors = {
            endError: ['ExtraOpenMissingClose',
                'Extra open brace or missing close brace'],
            closeError: ['ExtraCloseMissingOpen',
                'Extra close brace or missing open brace'],
            rightError: ['MissingLeftExtraRight',
                'Missing \\left or extra \\right']
        };
        this.kind = 'base';
        this.global = {};
        this.data = [];
        this.kind = 'base';
        if (this.isOpen) {
            this._env = {};
        }
        this.Push.apply(this, args);
    }
    Object.defineProperty(BaseItem.prototype, "env", {
        get: function () {
            return this._env;
        },
        set: function (value) {
            this._env = value;
        },
        enumerable: true,
        configurable: true
    });
    BaseItem.prototype.getProperty = function (key) {
        return this._properties[key] || this[key];
    };
    BaseItem.prototype.setProperty = function (key, value) {
        this._properties[key] = value;
        this[key] = value;
    };
    Object.defineProperty(BaseItem.prototype, "isOpen", {
        get: function () {
            return false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseItem.prototype, "isClose", {
        get: function () {
            return false;
        },
        enumerable: true,
        configurable: true
    });
    BaseItem.prototype.Push = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        TreeHelper_js_1.TreeHelper.printMethod('StackItem Push arguments: ' + this.data + ' arguments: ');
        TreeHelper_js_1.TreeHelper.printSimple(args.toString());
        this.data.push.apply(this.data, arguments);
    };
    BaseItem.prototype.Pop = function () {
        return this.data.pop();
    };
    BaseItem.prototype.getType = function () {
        return this.kind;
    };
    BaseItem.prototype.hasType = function (kind) {
        return kind === this.getType();
    };
    BaseItem.prototype.mmlData = function (inferred, forceRow) {
        TreeHelper_js_1.TreeHelper.printMethod('mmlData');
        if (inferred == null) {
            inferred = true;
        }
        if (this.data.length === 1 && !forceRow) {
            TreeHelper_js_1.TreeHelper.printSimple('End 1');
            return this.data[0];
        }
        var node = TreeHelper_js_1.TreeHelper.createNode('mrow', this.data, inferred ? { inferred: true } : {});
        return node;
    };
    BaseItem.prototype.checkItem = function (item) {
        TreeHelper_js_1.TreeHelper.printMethod('Checkitem base for ' + item.getType() + ' with ' + item);
        if (item.hasType('over') && this.isOpen) {
            item.setProperty('num', this.mmlData(false));
            this.data = [];
        }
        if (item.hasType('cell') && this.isOpen) {
            if (item.getProperty('linebreak')) {
                return false;
            }
            throw new TexError_js_1.default(['Misplaced', 'Misplaced %1', item.getName()]);
        }
        if (item.isClose && this.errors[item.getType() + 'Error']) {
            throw new TexError_js_1.default(this.errors[item.getType() + 'Error']);
        }
        if (!item.getProperty('isNotStack')) {
            return true;
        }
        this.Push(item.data[0]);
        return false;
    };
    BaseItem.prototype.With = function (def) {
        for (var id in def) {
            if (def.hasOwnProperty(id)) {
                this.setProperty(id, def[id]);
            }
        }
        return this;
    };
    BaseItem.prototype.toString = function () {
        return this.getType() + '[' + this.data.join('; ') + ']';
    };
    BaseItem.prototype.getName = function () {
        return this.getProperty('name');
    };
    return BaseItem;
}());
exports.BaseItem = BaseItem;
var StartItem = (function (_super) {
    __extends(StartItem, _super);
    function StartItem(global) {
        var _this = _super.call(this) || this;
        _this.kind = 'start';
        _this.global = global;
        return _this;
    }
    Object.defineProperty(StartItem.prototype, "isOpen", {
        get: function () {
            return true;
        },
        enumerable: true,
        configurable: true
    });
    StartItem.prototype.checkItem = function (item) {
        TreeHelper_js_1.TreeHelper.printMethod('Checkitem start');
        if (item.hasType('stop')) {
            return new MmlItem(this.mmlData());
        }
        return _super.prototype.checkItem.call(this, item);
    };
    return StartItem;
}(BaseItem));
exports.StartItem = StartItem;
var StopItem = (function (_super) {
    __extends(StopItem, _super);
    function StopItem() {
        var _this = _super.call(this) || this;
        _this.kind = 'stop';
        return _this;
    }
    Object.defineProperty(StopItem.prototype, "isClose", {
        get: function () {
            return true;
        },
        enumerable: true,
        configurable: true
    });
    return StopItem;
}(BaseItem));
exports.StopItem = StopItem;
var OpenItem = (function (_super) {
    __extends(OpenItem, _super);
    function OpenItem() {
        var _this = _super.call(this) || this;
        _this.errors['stopError'] = ['ExtraOpenMissingClose',
            'Extra open brace or missing close brace'];
        _this.kind = 'open';
        return _this;
    }
    Object.defineProperty(OpenItem.prototype, "isOpen", {
        get: function () {
            return true;
        },
        enumerable: true,
        configurable: true
    });
    OpenItem.prototype.checkItem = function (item) {
        TreeHelper_js_1.TreeHelper.printMethod('Checkitem open');
        if (item.hasType('close')) {
            var mml = this.mmlData();
            mml = TreeHelper_js_1.TreeHelper.cleanSubSup(mml);
            var node = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [mml], {});
            return new MmlItem(node);
        }
        return _super.prototype.checkItem.call(this, item);
    };
    return OpenItem;
}(BaseItem));
exports.OpenItem = OpenItem;
var CloseItem = (function (_super) {
    __extends(CloseItem, _super);
    function CloseItem() {
        var _this = _super.call(this) || this;
        _this.kind = 'close';
        return _this;
    }
    Object.defineProperty(CloseItem.prototype, "isClose", {
        get: function () {
            return true;
        },
        enumerable: true,
        configurable: true
    });
    return CloseItem;
}(BaseItem));
exports.CloseItem = CloseItem;
var PrimeItem = (function (_super) {
    __extends(PrimeItem, _super);
    function PrimeItem() {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var _this = _super.apply(this, __spread(args)) || this;
        _this.kind = 'prime';
        return _this;
    }
    PrimeItem.prototype.checkItem = function (item) {
        TreeHelper_js_1.TreeHelper.printMethod('Checkitem prime');
        if (!TreeHelper_js_1.TreeHelper.isType(this.data[0], 'msubsup')) {
            var node = TreeHelper_js_1.TreeHelper.createNode('msup', [this.data[0], this.data[1]], {});
            return [node, item];
        }
        TreeHelper_js_1.TreeHelper.setData(this.data[0], this.data[0].sup, this.data[1]);
        return [this.data[0], item];
    };
    return PrimeItem;
}(BaseItem));
exports.PrimeItem = PrimeItem;
var SubsupItem = (function (_super) {
    __extends(SubsupItem, _super);
    function SubsupItem() {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var _this = _super.apply(this, __spread(args)) || this;
        _this.kind = 'subsup';
        _this.errors['stopError'] = ['MissingScript',
            'Missing superscript or subscript argument'];
        _this.errors['supError'] = ['MissingOpenForSup',
            'Missing open brace for superscript'];
        _this.errors['subError'] = ['MissingOpenForSub',
            'Missing open brace for subscript'];
        return _this;
    }
    SubsupItem.prototype.checkItem = function (item) {
        TreeHelper_js_1.TreeHelper.printMethod('Checkitem subsup');
        if (item.hasType('open') || item.hasType('left')) {
            return true;
        }
        if (item.hasType('mml')) {
            if (this.getProperty('primes')) {
                if (this.getProperty('position') !== 2) {
                    TreeHelper_js_1.TreeHelper.setData(this.data[0], 2, this.getProperty('primes'));
                }
                else {
                    TreeHelper_js_1.TreeHelper.setProperties(this.getProperty('primes'), { variantForm: true });
                    var node = TreeHelper_js_1.TreeHelper.createNode('mrow', [this.getProperty('primes'), item.data[0]], {});
                    item.data[0] = node;
                }
            }
            TreeHelper_js_1.TreeHelper.setData(this.data[0], this.getProperty('position'), item.data[0]);
            if (this.getProperty('movesupsub') != null) {
                TreeHelper_js_1.TreeHelper.setProperties(this.data[0], { movesupsub: this.getProperty('movesupsub') });
            }
            var result = new MmlItem(this.data[0]);
            return result;
        }
        if (_super.prototype.checkItem.call(this, item)) {
            throw new TexError_js_1.default(this.errors[['', 'subError', 'supError'][this.getProperty('position')]]);
        }
    };
    SubsupItem.prototype.Pop = function () { };
    return SubsupItem;
}(BaseItem));
exports.SubsupItem = SubsupItem;
var OverItem = (function (_super) {
    __extends(OverItem, _super);
    function OverItem() {
        var _this = _super.call(this) || this;
        _this.kind = 'over';
        _this.setProperty('name', '\\over');
        return _this;
    }
    Object.defineProperty(OverItem.prototype, "isClose", {
        get: function () {
            return true;
        },
        enumerable: true,
        configurable: true
    });
    OverItem.prototype.checkItem = function (item) {
        TreeHelper_js_1.TreeHelper.printMethod('Checkitem over');
        if (item.hasType('over')) {
            throw new TexError_js_1.default(['AmbiguousUseOf', 'Ambiguous use of %1', item.getName()]);
        }
        if (item.isClose) {
            var mml = TreeHelper_js_1.TreeHelper.createNode('mfrac', [this.getProperty('num'), this.mmlData(false)], {});
            if (this.getProperty('thickness') != null) {
                TreeHelper_js_1.TreeHelper.setAttribute(mml, 'linethickness', this.getProperty('thickness'));
            }
            if (this.getProperty('open') || this.getProperty('close')) {
                TreeHelper_js_1.TreeHelper.setProperties(mml, { 'texWithDelims': true });
                mml = ParserUtil_js_1.ParserUtil.fixedFence(this.getProperty('open'), mml, this.getProperty('close'));
            }
            return [new MmlItem(mml), item];
        }
        return _super.prototype.checkItem.call(this, item);
    };
    OverItem.prototype.toString = function () { return 'over[' + this.getProperty('num') + ' / ' + this.data.join('; ') + ']'; };
    return OverItem;
}(BaseItem));
exports.OverItem = OverItem;
var LeftItem = (function (_super) {
    __extends(LeftItem, _super);
    function LeftItem() {
        var _this = _super.call(this) || this;
        _this.kind = 'left';
        _this.setProperty('delim', '('),
            _this.errors['stopError'] = ['ExtraLeftMissingRight',
                'Extra \\left or missing \\right'];
        return _this;
    }
    Object.defineProperty(LeftItem.prototype, "isOpen", {
        get: function () {
            return true;
        },
        enumerable: true,
        configurable: true
    });
    LeftItem.prototype.checkItem = function (item) {
        TreeHelper_js_1.TreeHelper.printMethod('Checkitem left');
        if (item.hasType('right')) {
            return new MmlItem(ParserUtil_js_1.ParserUtil.fenced(this.getProperty('delim'), this.mmlData(), item.getProperty('delim')));
        }
        return _super.prototype.checkItem.call(this, item);
    };
    return LeftItem;
}(BaseItem));
exports.LeftItem = LeftItem;
var RightItem = (function (_super) {
    __extends(RightItem, _super);
    function RightItem() {
        var _this = _super.call(this) || this;
        _this.kind = 'right';
        _this.setProperty('delim', ')');
        return _this;
    }
    Object.defineProperty(RightItem.prototype, "isClose", {
        get: function () {
            return true;
        },
        enumerable: true,
        configurable: true
    });
    return RightItem;
}(BaseItem));
exports.RightItem = RightItem;
var BeginItem = (function (_super) {
    __extends(BeginItem, _super);
    function BeginItem() {
        var _this = _super.call(this) || this;
        _this.kind = 'begin';
        return _this;
    }
    Object.defineProperty(BeginItem.prototype, "isOpen", {
        get: function () {
            return true;
        },
        enumerable: true,
        configurable: true
    });
    BeginItem.prototype.checkItem = function (item) {
        TreeHelper_js_1.TreeHelper.printMethod('Checkitem begin');
        if (item.hasType('end')) {
            if (item.getName() !== this.getName()) {
                throw new TexError_js_1.default(['EnvBadEnd', '\\begin{%1} ended with \\end{%2}',
                    this.getName(), item.getName()]);
            }
            if (!this.getProperty('end')) {
                return new MmlItem(this.mmlData());
            }
            return;
        }
        if (item.hasType('stop')) {
            throw new TexError_js_1.default(['EnvMissingEnd', 'Missing \\end{%1}', this.getName()]);
        }
        return _super.prototype.checkItem.call(this, item);
    };
    return BeginItem;
}(BaseItem));
exports.BeginItem = BeginItem;
var EndItem = (function (_super) {
    __extends(EndItem, _super);
    function EndItem() {
        var _this = _super.call(this) || this;
        _this.kind = 'end';
        return _this;
    }
    Object.defineProperty(EndItem.prototype, "isClose", {
        get: function () {
            return true;
        },
        enumerable: true,
        configurable: true
    });
    return EndItem;
}(BaseItem));
exports.EndItem = EndItem;
var StyleItem = (function (_super) {
    __extends(StyleItem, _super);
    function StyleItem() {
        var _this = _super.call(this) || this;
        _this.kind = 'style';
        return _this;
    }
    StyleItem.prototype.checkItem = function (item) {
        TreeHelper_js_1.TreeHelper.printMethod('Checkitem style');
        if (!item.isClose) {
            return _super.prototype.checkItem.call(this, item);
        }
        var mml = TreeHelper_js_1.TreeHelper.createNode('mstyle', this.data, this.getProperty('styles'));
        return [new MmlItem(mml), item];
    };
    return StyleItem;
}(BaseItem));
exports.StyleItem = StyleItem;
var PositionItem = (function (_super) {
    __extends(PositionItem, _super);
    function PositionItem() {
        var _this = _super.call(this) || this;
        _this.kind = 'position';
        return _this;
    }
    PositionItem.prototype.checkItem = function (item) {
        TreeHelper_js_1.TreeHelper.printMethod('Checkitem position');
        if (item.isClose) {
            throw new TexError_js_1.default(['MissingBoxFor', 'Missing box for %1', this.getName()]);
        }
        if (item.getProperty('isNotStack')) {
            var mml = item.mmlData();
            switch (this.getProperty('move')) {
                case 'vertical':
                    mml = TreeHelper_js_1.TreeHelper.createNode('mpadded', [mml], { height: this.getProperty('dh'), depth: this.getProperty('dd'),
                        voffset: this.getProperty('dh') });
                    return [new MmlItem(mml)];
                case 'horizontal':
                    return [new MmlItem(this.getProperty('left')), item,
                        new MmlItem(this.getProperty('right'))];
            }
        }
        return _super.prototype.checkItem.call(this, item);
    };
    return PositionItem;
}(BaseItem));
exports.PositionItem = PositionItem;
var ArrayItem = (function (_super) {
    __extends(ArrayItem, _super);
    function ArrayItem() {
        var _this = _super.call(this) || this;
        _this.table = [];
        _this.row = [];
        _this.frame = [];
        _this.hfill = [];
        _this.copyEnv = false;
        _this.arraydef = {};
        _this.dashed = false;
        _this.kind = 'array';
        return _this;
    }
    Object.defineProperty(ArrayItem.prototype, "isOpen", {
        get: function () {
            return true;
        },
        enumerable: true,
        configurable: true
    });
    ArrayItem.prototype.checkItem = function (item) {
        TreeHelper_js_1.TreeHelper.printMethod('Checkitem array');
        if (item.isClose && !item.hasType('over')) {
            if (item.getProperty('isEntry')) {
                this.EndEntry();
                this.clearEnv();
                return false;
            }
            if (item.getProperty('isCR')) {
                this.EndEntry();
                this.EndRow();
                this.clearEnv();
                return false;
            }
            this.EndTable();
            this.clearEnv();
            var scriptlevel = this.arraydef['scriptlevel'];
            delete this.arraydef['scriptlevel'];
            var mml = TreeHelper_js_1.TreeHelper.createNode('mtable', this.table, this.arraydef);
            if (this.frame.length === 4) {
                TreeHelper_js_1.TreeHelper.setAttribute(mml, 'frame', this.dashed ? 'dashed' : 'solid');
            }
            else if (this.frame.length) {
                if (this.arraydef['rowlines']) {
                    this.arraydef['rowlines'] =
                        this.arraydef['rowlines'].replace(/none( none) + $/, 'none');
                }
                mml = TreeHelper_js_1.TreeHelper.createNode('menclose', [mml], { notation: this.frame.join(' '), isFrame: true });
                if ((this.arraydef['columnlines'] || 'none') != 'none' ||
                    (this.arraydef['rowlines'] || 'none') != 'none') {
                    TreeHelper_js_1.TreeHelper.setAttribute(mml, 'padding', 0);
                }
            }
            if (scriptlevel) {
                TreeHelper_js_1.TreeHelper.untested(3);
                mml = TreeHelper_js_1.TreeHelper.createNode('mstyle', [mml], { scriptlevel: scriptlevel });
            }
            if (this.getProperty('open') || this.getProperty('close')) {
                mml = ParserUtil_js_1.ParserUtil.fenced(this.getProperty('open'), mml, this.getProperty('close'));
            }
            var newItem = new MmlItem(mml);
            if (this.getProperty('requireClose')) {
                if (item.hasType('close')) {
                    return newItem;
                }
                throw new TexError_js_1.default(['MissingCloseBrace', 'Missing close brace']);
            }
            return [newItem, item];
        }
        return _super.prototype.checkItem.call(this, item);
    };
    ArrayItem.prototype.EndEntry = function () {
        var mtd = TreeHelper_js_1.TreeHelper.createNode('mtd', this.data, {});
        if (this.hfill.length) {
            if (this.hfill[0] === 0) {
                TreeHelper_js_1.TreeHelper.setAttribute(mtd, 'columnalign', 'right');
            }
            if (this.hfill[this.hfill.length - 1] === this.data.length) {
                TreeHelper_js_1.TreeHelper.setAttribute(mtd, 'columnalign', TreeHelper_js_1.TreeHelper.getAttribute(mtd, 'columnalign') ? 'center' : 'left');
            }
        }
        this.row.push(mtd);
        this.data = [];
        this.hfill = [];
    };
    ArrayItem.prototype.EndRow = function () {
        if (this.getProperty('isNumbered') && this.row.length === 3) {
            this.row.unshift(this.row.pop());
            var node = TreeHelper_js_1.TreeHelper.createNode('mlabeledtr', this.row, {});
        }
        else {
            node = TreeHelper_js_1.TreeHelper.createNode('mtr', this.row, {});
        }
        this.table.push(node);
        this.row = [];
    };
    ArrayItem.prototype.EndTable = function () {
        if (this.data.length || this.row.length) {
            this.EndEntry();
            this.EndRow();
        }
        this.checkLines();
    };
    ArrayItem.prototype.checkLines = function () {
        if (this.arraydef['rowlines']) {
            var lines = this.arraydef['rowlines'].split(/ /);
            if (lines.length === this.table.length) {
                this.frame.push('bottom');
                lines.pop();
                this.arraydef['rowlines'] = lines.join(' ');
            }
            else if (lines.length < this.table.length - 1) {
                this.arraydef['rowlines'] += ' none';
            }
        }
        if (this.getProperty('rowspacing')) {
            var rows = this.arraydef['rowspacing'].split(/ /);
            while (rows.length < this.table.length) {
                rows.push(this.getProperty('rowspacing') + 'em');
            }
            this.arraydef['rowspacing'] = rows.join(' ');
        }
    };
    ArrayItem.prototype.clearEnv = function () {
        for (var id in this.env) {
            if (this.env.hasOwnProperty(id)) {
                delete this.env[id];
            }
        }
    };
    return ArrayItem;
}(BaseItem));
exports.ArrayItem = ArrayItem;
var CellItem = (function (_super) {
    __extends(CellItem, _super);
    function CellItem() {
        var _this = _super.call(this) || this;
        _this.kind = 'cell';
        return _this;
    }
    Object.defineProperty(CellItem.prototype, "isClose", {
        get: function () {
            return true;
        },
        enumerable: true,
        configurable: true
    });
    return CellItem;
}(BaseItem));
exports.CellItem = CellItem;
var MmlItem = (function (_super) {
    __extends(MmlItem, _super);
    function MmlItem() {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var _this = _super.apply(this, __spread(args)) || this;
        _this.kind = 'mml';
        _this.setProperty('isNotStack', true);
        return _this;
    }
    MmlItem.prototype.Add = function () {
        this.data.push.apply(this.data, arguments);
        return this;
    };
    return MmlItem;
}(BaseItem));
exports.MmlItem = MmlItem;
var FnItem = (function (_super) {
    __extends(FnItem, _super);
    function FnItem() {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var _this = _super.apply(this, __spread(args)) || this;
        _this.kind = 'fn';
        return _this;
    }
    FnItem.prototype.checkItem = function (item) {
        TreeHelper_js_1.TreeHelper.printMethod('Checkitem fn');
        if (this.data[0]) {
            TreeHelper_js_1.TreeHelper.printSimple('case 1');
            if (item.isOpen) {
                TreeHelper_js_1.TreeHelper.printSimple('case 2');
                return true;
            }
            if (!item.hasType('fn')) {
                TreeHelper_js_1.TreeHelper.printSimple('case 3');
                if (!item.hasType('mml') || !item.data[0]) {
                    TreeHelper_js_1.TreeHelper.printSimple('case 4');
                    return [this.data[0], item];
                }
                if (TreeHelper_js_1.TreeHelper.isType(item.data[0], 'mspace')) {
                    TreeHelper_js_1.TreeHelper.untested(100);
                    return [this.data[0], item];
                }
                var mml = item.data[0];
                if (TreeHelper_js_1.TreeHelper.isEmbellished(mml)) {
                    TreeHelper_js_1.TreeHelper.printSimple('case 5');
                    mml = TreeHelper_js_1.TreeHelper.getCoreMO(mml);
                }
                if ([0, 0, 1, 1, 0, 1, 1, 0, 0, 0][TreeHelper_js_1.TreeHelper.getTexClass(mml)]) {
                    return [this.data[0], item];
                }
            }
            var text = TreeHelper_js_1.TreeHelper.createText(MmlEntities_js_1.MmlEntities.ENTITIES.ApplyFunction);
            var node = TreeHelper_js_1.TreeHelper.createNode('mo', [], { texClass: MmlNode_js_1.TEXCLASS.NONE }, text);
            return [this.data[0], node, item];
        }
        return _super.prototype.checkItem.apply(this, arguments);
    };
    return FnItem;
}(BaseItem));
exports.FnItem = FnItem;
var NotItem = (function (_super) {
    __extends(NotItem, _super);
    function NotItem() {
        var _this = _super.call(this) || this;
        _this.remap = MapHandler_js_1.default.getInstance().getMap('not_remap');
        _this.kind = 'not';
        return _this;
    }
    NotItem.prototype.checkItem = function (item) {
        TreeHelper_js_1.TreeHelper.printMethod('Checkitem not');
        var mml, c;
        if (item.hasType('open') || item.hasType('left')) {
            return true;
        }
        if (item.hasType('mml') &&
            (TreeHelper_js_1.TreeHelper.isType(item.data[0], 'mo') || TreeHelper_js_1.TreeHelper.isType(item.data[0], 'mi') ||
                TreeHelper_js_1.TreeHelper.isType(item.data[0], 'mtext'))) {
            mml = item.data[0];
            TreeHelper_js_1.TreeHelper.printJSON(mml);
            c = TreeHelper_js_1.TreeHelper.getText(mml);
            if (c.length === 1 && !TreeHelper_js_1.TreeHelper.getProperty(mml, 'movesupsub') &&
                TreeHelper_js_1.TreeHelper.getChildren(mml).length === 1) {
                if (this.remap.contains(c)) {
                    var textNode = TreeHelper_js_1.TreeHelper.createText(this.remap.lookup(c).char);
                    TreeHelper_js_1.TreeHelper.setData(mml, 0, textNode);
                }
                else {
                    textNode = TreeHelper_js_1.TreeHelper.createText('\u0338');
                    TreeHelper_js_1.TreeHelper.appendChildren(mml, [textNode]);
                }
                return item;
            }
        }
        textNode = TreeHelper_js_1.TreeHelper.createText('\u29F8');
        var mtextNode = TreeHelper_js_1.TreeHelper.createNode('mtext', [], {}, textNode);
        var paddedNode = TreeHelper_js_1.TreeHelper.createNode('mpadded', [mtextNode], { width: 0 });
        mml = TreeHelper_js_1.TreeHelper.createNode('TeXAtom', [paddedNode], { texClass: MmlNode_js_1.TEXCLASS.REL });
        return [mml, item];
    };
    return NotItem;
}(BaseItem));
exports.NotItem = NotItem;
var DotsItem = (function (_super) {
    __extends(DotsItem, _super);
    function DotsItem() {
        var _this = _super.call(this) || this;
        _this.kind = 'dots';
        return _this;
    }
    DotsItem.prototype.checkItem = function (item) {
        TreeHelper_js_1.TreeHelper.printMethod('Checkitem dots');
        if (item.hasType('open') || item.hasType('left')) {
            return true;
        }
        var dots = this.getProperty('ldots');
        if (item.hasType('mml') && TreeHelper_js_1.TreeHelper.isEmbellished(item.data[0])) {
            var tclass = TreeHelper_js_1.TreeHelper.getTexClass(TreeHelper_js_1.TreeHelper.getCoreMO(item.data[0]));
            if (tclass === MmlNode_js_1.TEXCLASS.BIN || tclass === MmlNode_js_1.TEXCLASS.REL) {
                dots = this.getProperty('cdots');
            }
        }
        return [dots, item];
    };
    return DotsItem;
}(BaseItem));
exports.DotsItem = DotsItem;
var AMSarrayItem = (function (_super) {
    __extends(AMSarrayItem, _super);
    function AMSarrayItem(name, numbered, taggable, global) {
        var _this = _super.call(this) || this;
        _this.numbered = false;
        _this.save = {};
        _this.kind = 'AMSarray';
        _this.numbered = numbered;
        _this.save['notags'] = global['notags'];
        _this.save['notag'] = global['notag'];
        global['notags'] = (taggable ? null : name);
        global['tagged'] = !numbered && !global['forcetag'];
        return _this;
    }
    AMSarrayItem.prototype.autoTag = function () { };
    AMSarrayItem.prototype.getTag = function () {
        return;
    };
    AMSarrayItem.prototype.clearTag = function () { };
    AMSarrayItem.prototype.EndEntry = function () {
        TreeHelper_js_1.TreeHelper.printMethod('AMS-EndEntry');
        if (this.row.length) {
            ParserUtil_js_1.ParserUtil.fixInitialMO(this.data);
        }
        var node = TreeHelper_js_1.TreeHelper.createNode('mtd', this.data, {});
        this.row.push(node);
        this.data = [];
    };
    AMSarrayItem.prototype.EndRow = function () {
        TreeHelper_js_1.TreeHelper.printMethod('AMS-EndRow');
        var mtr = 'mtr';
        if (!this.global['tag'] && this.numbered) {
            this.autoTag();
        }
        if (this.global['tag'] && !this.global['notags']) {
            this.row = [this.getTag()].concat(this.row);
            mtr = 'mlabeledtr';
        }
        else {
            this.clearTag();
        }
        if (this.numbered) {
            delete this.global['notag'];
        }
        var node = TreeHelper_js_1.TreeHelper.createNode(mtr, this.row, {});
        this.table.push(node);
        this.row = [];
    };
    AMSarrayItem.prototype.EndTable = function () {
        TreeHelper_js_1.TreeHelper.printMethod('AMS-EndTable');
        _super.prototype.EndTable.call(this);
        this.global['notags'] = this.save['notags'];
        this.global['notag'] = this.save['notag'];
    };
    return AMSarrayItem;
}(ArrayItem));
exports.AMSarrayItem = AMSarrayItem;
//# sourceMappingURL=StackItem.js.map