import { Attributes, Args, ParseMethod, ParseInput, ParseResult } from './Types.js';
import { Symbol, Macro } from './Symbol.js';
export interface SymbolMap {
    name: string;
    parser: ParseMethod;
    contains(symbol: string): boolean;
    parserFor(symbol: string): ParseMethod;
    parse([symbol, env]: ParseInput): ParseResult;
}
export declare abstract class AbstractSymbolMap<T> implements SymbolMap {
    private _name;
    private _parser;
    constructor(_name: string);
    readonly name: string;
    abstract contains(symbol: string): boolean;
    parserFor(symbol: string): ParseMethod;
    parse([symbol, env]: ParseInput): any;
    parser: ParseMethod;
    abstract lookup(symbol: string): T;
}
export declare class RegExpMap extends AbstractSymbolMap<string> {
    private _regExp;
    static create(name: string, parser: ParseMethod, regexp: RegExp): RegExpMap;
    constructor(name: string, _regExp: RegExp);
    contains(symbol: string): boolean;
    lookup(symbol: string): string;
}
export declare abstract class AbstractParseMap<K> extends AbstractSymbolMap<K> {
    private map;
    lookup(symbol: string): K;
    contains(symbol: string): boolean;
    protected add(symbol: string, object: K): void;
    abstract addElement<K>(symbol: string, object: K): void;
}
export declare class CharacterMap extends AbstractParseMap<Symbol> {
    protected static addCharacters(map: CharacterMap, json: {
        [index: string]: string | [string, Attributes];
    }): void;
    static create(name: string, parser: ParseMethod, json: {
        [index: string]: string | [string, Attributes];
    }): CharacterMap;
    addElement(symbol: string, pair: [string, null] | [string, Attributes]): void;
}
export declare class DelimiterMap extends CharacterMap {
    static create(name: string, parser: ParseMethod, json: {
        [index: string]: string | [string, Attributes];
    }): DelimiterMap;
    parse([symbol, env]: ParseInput): any;
}
export declare class MacroMap extends AbstractParseMap<Macro> {
    private _functionMap;
    protected static addCommands(map: MacroMap, json: {
        [index: string]: string | Args[];
    }): void;
    static create(name: string, json: {
        [index: string]: string | Args[];
    }): MacroMap;
    functionMap: Record<string, ParseMethod>;
    parserFor(symbol: string): ParseMethod;
    addElement(symbol: string, object: Args[]): void;
}
export declare class CommandMap extends MacroMap {
    static create(name: string, json: {
        [index: string]: string | Args[];
    }): MacroMap;
    parse([symbol, env]: ParseInput): any;
}
export declare class EnvironmentMap extends MacroMap {
    static create(name: string, json: {
        [index: string]: string | Args[];
    }): MacroMap;
    parse([symbol, env]: ParseInput): boolean;
}
