"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var TreeHelper_js_1 = require("./TreeHelper.js");
var OldParser_js_1 = require("./OldParser.js");
var ParserUtil_js_1 = require("./ParserUtil.js");
var ParseMethods_js_1 = require("./ParseMethods.js");
var TexError_js_1 = require("./TexError.js");
var NewTex;
(function (NewTex) {
    NewTex.display = false;
    function Compile(tex, display) {
        var script = {
            type: "math/tex" + (display ? "; mode=display" : ""),
            innerText: tex,
            MathJax: {}
        };
        var node = Translate(script, [], {});
        node.setInheritedAttributes();
        return node;
    }
    NewTex.Compile = Compile;
    var formatError = function (err, math, display, script) {
        var message = err.message.replace(/\n.*/, "");
        return TreeHelper_js_1.TreeHelper.createError(message);
    };
    function Translate(script, configurations, stackitem) {
        if (configurations === void 0) { configurations = []; }
        TreeHelper_js_1.TreeHelper.printMethod('Translate');
        TreeHelper_js_1.TreeHelper.printSimple(script.toString());
        var mml;
        var isError = false;
        var math = script.innerText;
        NewTex.display = (script.type.replace(/\n/g, " ").match(/(;|\s|\n)mode\s*=\s*display(;|\s|\n|$)/) != null);
        try {
            mml = new OldParser_js_1.OldParser(math, null, ParseMethods_js_1.ParseMethods).mml();
            TreeHelper_js_1.TreeHelper.printSimple(mml.toString());
        }
        catch (err) {
            if (!(err instanceof TexError_js_1.default)) {
                throw err;
            }
            mml = formatError(err, math, NewTex.display, script);
            isError = true;
        }
        mml = TreeHelper_js_1.TreeHelper.cleanSubSup(mml);
        if (TreeHelper_js_1.TreeHelper.isType(mml, 'mtable') &&
            TreeHelper_js_1.TreeHelper.getAttribute(mml, 'displaystyle') === 'inherit') {
            TreeHelper_js_1.TreeHelper.untested('Tagged equations');
            TreeHelper_js_1.TreeHelper.setAttribute(mml, 'displaystyle', NewTex.display);
        }
        var mathNode = TreeHelper_js_1.TreeHelper.createMath(mml);
        var root = TreeHelper_js_1.TreeHelper.getRoot(mathNode);
        if (NewTex.display) {
            TreeHelper_js_1.TreeHelper.setAttribute(root, 'display', 'block');
        }
        ParserUtil_js_1.ParserUtil.combineRelations(root);
        return mathNode;
    }
    NewTex.Translate = Translate;
})(NewTex = exports.NewTex || (exports.NewTex = {}));
//# sourceMappingURL=Translate.js.map