"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var MmlNode_js_1 = require("../../core/MmlTree/MmlNode.js");
var MmlFactory_js_1 = require("../../core/MmlTree/MmlFactory.js");
var JsonMmlVisitor_js_1 = require("../../core/MmlTree/JsonMmlVisitor.js");
var TreeHelper;
(function (TreeHelper) {
    var factory = new MmlFactory_js_1.MmlFactory();
    var visitor = new JsonMmlVisitor_js_1.JsonMmlVisitor();
    var attrs = ['autoOP',
        'fnOP',
        'movesupsub',
        'subsupOK',
        'texprimestyle',
        'useHeight',
        'variantForm',
        'texWithDelims',
        'open',
        'close'
    ];
    var methodOut = false;
    var defOut = false;
    var jsonOut = false;
    var simpleOut = false;
    function createNode(kind, children, def, text) {
        var node = factory.create(kind, def, children);
        if (text) {
            node.appendChild(text);
        }
        setProperties(node, def);
        printJSON(node);
        return node;
    }
    TreeHelper.createNode = createNode;
    ;
    function createText(text) {
        if (text == null) {
            return null;
        }
        var node = factory.create('text').setText(text);
        printJSON(node);
        return node;
    }
    TreeHelper.createText = createText;
    ;
    function createEntity(code) {
        return createText(String.fromCharCode(parseInt(code, 16)));
    }
    TreeHelper.createEntity = createEntity;
    ;
    function createError(message) {
        var text = createText(message);
        var mtext = createNode('mtext', [], {}, text);
        var error = createNode('merror', [mtext], {});
        return error;
    }
    TreeHelper.createError = createError;
    ;
    function createMath(math) {
        var mathNode;
        if (math.isInferred) {
            mathNode = createNode('math', [math], {});
        }
        else {
            if (math.isKind('mrow') && !math.isKind('math')) {
                mathNode = createNode('math', [], {});
                var inferredMrow = mathNode.childNodes[0];
                inferredMrow.attributes = math.attributes;
                setProperties(inferredMrow, math.getAllProperties());
                mathNode.setChildren(math.childNodes);
            }
            else if (!math.isKind('math')) {
                mathNode = createNode('math', [math], {});
            }
            else {
                mathNode = math;
            }
        }
        return mathNode;
    }
    TreeHelper.createMath = createMath;
    ;
    function getRoot(tree) {
        return tree;
    }
    TreeHelper.getRoot = getRoot;
    ;
    function getChildren(node) {
        return node.childNodes;
    }
    TreeHelper.getChildren = getChildren;
    ;
    function getText(node) {
        return node.getText();
    }
    TreeHelper.getText = getText;
    ;
    function appendChildren(node, children) {
        try {
            for (var children_1 = __values(children), children_1_1 = children_1.next(); !children_1_1.done; children_1_1 = children_1.next()) {
                var child = children_1_1.value;
                node.appendChild(child);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (children_1_1 && !children_1_1.done && (_a = children_1.return)) _a.call(children_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        var e_1, _a;
    }
    TreeHelper.appendChildren = appendChildren;
    ;
    function setAttribute(node, attribute, value) {
        node.attributes.set(attribute, value);
    }
    TreeHelper.setAttribute = setAttribute;
    ;
    function setProperties(node, properties) {
        printDef(properties);
        try {
            for (var _a = __values(Object.keys(properties)), _b = _a.next(); !_b.done; _b = _a.next()) {
                var name_1 = _b.value;
                var value = properties[name_1];
                if (name_1 === 'texClass') {
                    node.texClass = value;
                }
                else if (name_1 === 'movablelimits') {
                    node.setProperty('movablelimits', value);
                    if (node.isKind('mo') || node.isKind('mstyle')) {
                        node.attributes.set('movablelimits', value);
                    }
                }
                else if (name_1 === 'inferred') {
                }
                else if (attrs.indexOf(name_1) !== -1) {
                    node.setProperty(name_1, value);
                }
                else {
                    var inherited = node.attributes.getInherited(name_1);
                    if (inherited !== value) {
                        node.attributes.set(name_1, value);
                    }
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
            }
            finally { if (e_2) throw e_2.error; }
        }
        var e_2, _c;
    }
    TreeHelper.setProperties = setProperties;
    ;
    function getProperty(node, property) {
        return node.getProperty(property);
    }
    TreeHelper.getProperty = getProperty;
    ;
    function getAttribute(node, attr) {
        return node.attributes.get(attr);
    }
    TreeHelper.getAttribute = getAttribute;
    function getChildAt(node, position) {
        return node.childNodes[position];
    }
    TreeHelper.getChildAt = getChildAt;
    ;
    function setData(node, position, child) {
        var children = node.childNodes;
        children[position] = child;
        if (child) {
            child.parent = node;
        }
        node.setTeXclass(null);
    }
    TreeHelper.setData = setData;
    ;
    function copyChildren(oldNode, newNode) {
        var children = oldNode.childNodes;
        for (var i = 0; i < children.length; i++) {
            setData(newNode, i, children[i]);
        }
    }
    TreeHelper.copyChildren = copyChildren;
    ;
    function copyAttributes(oldNode, newNode) {
        newNode.attributes = oldNode.attributes;
        setProperties(newNode, oldNode.getAllProperties());
    }
    TreeHelper.copyAttributes = copyAttributes;
    ;
    function isType(node, kind) {
        return node.isKind(kind);
    }
    TreeHelper.isType = isType;
    ;
    function isEmbellished(node) {
        return node.isEmbellished;
    }
    TreeHelper.isEmbellished = isEmbellished;
    ;
    function getTexClass(node) {
        return node.texClass;
    }
    TreeHelper.getTexClass = getTexClass;
    ;
    function getCore(node) {
        return node.core();
    }
    TreeHelper.getCore = getCore;
    ;
    function getCoreMO(node) {
        return node.coreMO();
    }
    TreeHelper.getCoreMO = getCoreMO;
    ;
    function cleanSubSup(node) {
        var rewrite = [];
        node.walkTree(function (n, d) {
            var children = n.childNodes;
            if ((n.isKind('msubsup') && (!children[n.sub] ||
                !children[n.sup])) ||
                (n.isKind('munderover') && (!children[n.under] ||
                    !children[n.over]))) {
                d.unshift(n);
            }
        }, rewrite);
        try {
            for (var rewrite_1 = __values(rewrite), rewrite_1_1 = rewrite_1.next(); !rewrite_1_1.done; rewrite_1_1 = rewrite_1.next()) {
                var n = rewrite_1_1.value;
                var children = n.childNodes;
                var parent_1 = n.parent;
                var ms = void 0, newNode = void 0;
                if (n.isKind('msubsup')) {
                    ms = n;
                    newNode = (children[ms.sub] ?
                        createNode('msub', [children[ms.base], children[ms.sub]], {}) :
                        createNode('msup', [children[ms.base], children[ms.sup]], {}));
                }
                else {
                    ms = n;
                    newNode = (children[ms.under] ?
                        createNode('munder', [children[ms.base], children[ms.under]], {}) :
                        createNode('mover', [children[ms.base], children[ms.over]], {}));
                }
                if (parent_1) {
                    parent_1.replaceChild(newNode, n);
                }
                else {
                    node = newNode;
                }
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (rewrite_1_1 && !rewrite_1_1.done && (_a = rewrite_1.return)) _a.call(rewrite_1);
            }
            finally { if (e_3) throw e_3.error; }
        }
        return node;
        var e_3, _a;
    }
    TreeHelper.cleanSubSup = cleanSubSup;
    ;
    function printSimple(txt) {
        if (simpleOut) {
            console.log(txt);
        }
    }
    TreeHelper.printSimple = printSimple;
    ;
    function untested(kind) {
        console.log('Untested case ' + kind);
    }
    TreeHelper.untested = untested;
    ;
    function printMethod(text) {
        if (methodOut) {
            console.log('In ' + text);
        }
    }
    TreeHelper.printMethod = printMethod;
    ;
    function printJSON(node) {
        if (jsonOut) {
            console.log(visitor.visitNode(node));
        }
    }
    TreeHelper.printJSON = printJSON;
    ;
    function printDef(def) {
        if (methodOut && defOut) {
            console.log('With:');
            for (var x in def) {
                console.log(x + ': ' + def[x]);
            }
        }
    }
    TreeHelper.printDef = printDef;
    ;
    function isNode(item) {
        return item instanceof MmlNode_js_1.AbstractMmlNode || item instanceof MmlNode_js_1.AbstractMmlEmptyNode;
    }
    TreeHelper.isNode = isNode;
    ;
    function isInferred(node) {
        return node.isInferred;
    }
    TreeHelper.isInferred = isInferred;
    ;
})(TreeHelper = exports.TreeHelper || (exports.TreeHelper = {}));
//# sourceMappingURL=TreeHelper.js.map