import { StringMap } from './Wrapper.js';
export declare type CharOptions = {
    c?: string;
    f?: string;
    css?: number;
    ic?: number;
    sk?: number;
};
export declare const enum CSS {
    width = 1,
    padding = 2,
    content = 4,
}
export declare type CharData = [number, number, number] | [number, number, number, CharOptions];
export declare type CharMap = {
    [n: number]: CharData;
};
export declare type CharMapMap = {
    [name: string]: CharMap;
};
export declare type VariantData = {
    linked: CharMap[];
    chars: CharMap;
    classes?: string;
};
export declare type VariantMap = {
    [name: string]: VariantData;
};
export declare const enum DIRECTION {
    None = 0,
    Vertical = 1,
    Horizontal = 2,
}
export declare const V: DIRECTION;
export declare const H: DIRECTION;
export declare type DelimiterData = {
    dir: DIRECTION;
    sizes?: number[];
    variants?: number[];
    schar?: number[];
    stretch?: number[];
    HDW?: number[];
    min?: number;
    c?: number;
};
export declare type DelimiterMap = {
    [n: number]: DelimiterData;
};
export declare const NOSTRETCH: DelimiterData;
export declare type FontParameters = {
    x_height: number;
    quad: number;
    num1: number;
    num2: number;
    num3: number;
    denom1: number;
    denom2: number;
    sup1: number;
    sup2: number;
    sup3: number;
    sub1: number;
    sub2: number;
    sup_drop: number;
    sub_drop: number;
    delim1: number;
    delim2: number;
    axis_height: number;
    rule_thickness: number;
    big_op_spacing1: number;
    big_op_spacing2: number;
    big_op_spacing3: number;
    big_op_spacing4: number;
    big_op_spacing5: number;
    surd_height: number;
    scriptspace: number;
    nulldelimiterspace: number;
    delimiterfactor: number;
    delimitershortfall: number;
    min_rule_thickness: number;
};
export declare class FontData {
    protected static defaultVariants: string[][];
    static defaultParams: FontParameters;
    protected static defaultDelimiters: DelimiterMap;
    protected static defaultChars: CharMapMap;
    protected static defaultSizeVariants: string[];
    protected static defaultVariantClasses: StringMap;
    protected variant: VariantMap;
    protected delimiters: DelimiterMap;
    protected sizeVariants: string[];
    params: FontParameters;
    constructor();
    createVariant(name: string, inherit?: string, link?: string): void;
    createVariants(variants: string[][]): void;
    defineChars(name: string, chars: CharMap): void;
    defineDelimiters(delims: DelimiterMap): void;
    getDelimiter(n: number): DelimiterData;
    getSizeVariant(n: number, i: number): string;
    getChar(name: string, n: number): CharData;
    getVariant(name: string): VariantData;
    char(n: number, escape?: boolean): string;
}
