import { AbstractWrapper } from '../../core/Tree/Wrapper.js';
import { PropertyList } from '../../core/Tree/Node.js';
import { MmlNode, TextNode } from '../../core/MmlTree/MmlNode.js';
import { Property } from '../../core/Tree/Node.js';
import { OptionList } from '../../util/Options.js';
import { CHTML } from '../chtml.js';
import { CHTMLWrapperFactory } from './WrapperFactory.js';
import { CHTMLmo } from './Wrappers/mo.js';
import { BBox } from './BBox.js';
import { FontData, DelimiterData, DIRECTION } from './FontData.js';
import { StyleList } from './CssStyles.js';
export declare type StringMap = {
    [key: string]: string;
};
export declare const FONTSIZE: StringMap;
export declare const SPACE: StringMap;
export declare class CHTMLWrapper extends AbstractWrapper<MmlNode, CHTMLWrapper> {
    static kind: string;
    static autoStyle: boolean;
    static styles: StyleList;
    static removeStyles: [string];
    static skipAttributes: {
        [name: string]: boolean;
    };
    static BOLDVARIANTS: {
        [name: string]: StringMap;
    };
    static ITALICVARIANTS: {
        [name: string]: StringMap;
    };
    protected factory: CHTMLWrapperFactory;
    parent: CHTMLWrapper;
    childNodes: CHTMLWrapper[];
    chtml: HTMLElement;
    protected removedStyles: StringMap;
    protected styles: CSSStyleDeclaration;
    variant: string;
    bbox: BBox;
    protected bboxComputed: boolean;
    stretch: DelimiterData;
    font: FontData;
    readonly CHTML: CHTML;
    readonly metrics: {
        em: number;
        ex: number;
        containerWidth: number;
        lineWidth: number;
        scale: number;
    };
    constructor(factory: CHTMLWrapperFactory, node: MmlNode, parent?: CHTMLWrapper);
    wrap(node: MmlNode, parent?: CHTMLWrapper): CHTMLWrapper;
    toCHTML(parent: HTMLElement): void;
    getBBox(save?: boolean): BBox;
    protected computeBBox(bbox: BBox): void;
    protected getStyles(): void;
    protected getVariant(): void;
    protected explicitVariant(fontFamily: string, fontWeight: string, fontStyle: string): string;
    protected getScale(): void;
    protected getSpace(): void;
    protected standardCHTMLnode(parent: HTMLElement): HTMLElement;
    protected createCHTMLnode(parent: HTMLElement): HTMLElement;
    protected handleStyles(): void;
    protected handleVariant(): void;
    protected handleScale(): void;
    protected setScale(chtml: HTMLElement, rscale: number): HTMLElement;
    protected handleSpace(): void;
    protected handleColor(): void;
    protected handleAttributes(): void;
    core(): CHTMLWrapper;
    coreMO(): CHTMLmo;
    getText(): string;
    canStretch(direction: DIRECTION): boolean;
    drawBBox(): void;
    protected percent(m: number): string;
    protected em(m: number): string;
    protected px(m: number, M?: number): string;
    protected length2em(length: Property, size?: number, scale?: number): number;
    protected unicodeChars(text: string): number[];
    protected char(n: number, escape?: boolean): string;
    html(type: string, def?: OptionList, content?: HTMLElement[]): HTMLElement;
    text(text: string): Text;
    mmlText(text: string): TextNode;
    mmlNode(kind: string, properties?: PropertyList, children?: MmlNode[]): MmlNode;
}
export declare type CHTMLWrapperClass = typeof CHTMLWrapper;
