"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var Wrapper_js_1 = require("../../core/Tree/Wrapper.js");
var MmlNode_js_1 = require("../../core/MmlTree/MmlNode.js");
var string_js_1 = require("../../util/string.js");
var LENGTHS = require("../../util/lengths.js");
var BBox_js_1 = require("./BBox.js");
var FontData_js_1 = require("./FontData.js");
exports.FONTSIZE = {
    '70.7%': 's',
    '70%': 's',
    '50%': 'ss',
    '60%': 'Tn',
    '85%': 'sm',
    '120%': 'lg',
    '144%': 'Lg',
    '173%': 'LG',
    '207%': 'hg',
    '249%': 'HG'
};
exports.SPACE = (_a = {},
    _a[LENGTHS.em(3 / 18)] = '1',
    _a[LENGTHS.em(4 / 18)] = '2',
    _a[LENGTHS.em(5 / 18)] = '3',
    _a);
var CHTMLWrapper = (function (_super) {
    __extends(CHTMLWrapper, _super);
    function CHTMLWrapper(factory, node, parent) {
        if (parent === void 0) { parent = null; }
        var _this = _super.call(this, factory, node) || this;
        _this.parent = null;
        _this.chtml = null;
        _this.removedStyles = null;
        _this.styles = null;
        _this.variant = '';
        _this.bboxComputed = false;
        _this.stretch = FontData_js_1.NOSTRETCH;
        _this.font = null;
        _this.parent = parent;
        _this.font = factory.chtml.font;
        _this.bbox = BBox_js_1.BBox.zero();
        _this.getStyles();
        _this.getVariant();
        _this.getScale();
        _this.getSpace();
        _this.childNodes = node.childNodes.map(function (child) {
            return _this.wrap(child);
        });
        return _this;
    }
    Object.defineProperty(CHTMLWrapper.prototype, "CHTML", {
        get: function () {
            return this.factory.chtml;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CHTMLWrapper.prototype, "metrics", {
        get: function () {
            return this.factory.chtml.math.metrics;
        },
        enumerable: true,
        configurable: true
    });
    CHTMLWrapper.prototype.wrap = function (node, parent) {
        if (parent === void 0) { parent = null; }
        var wrapped = this.factory.wrap(node, parent || this);
        if (parent) {
            parent.childNodes.push(wrapped);
        }
        this.CHTML.nodeMap.set(node, wrapped);
        return wrapped;
    };
    CHTMLWrapper.prototype.toCHTML = function (parent) {
        var chtml = this.standardCHTMLnode(parent);
        try {
            for (var _a = __values(this.childNodes), _b = _a.next(); !_b.done; _b = _a.next()) {
                var child = _b.value;
                child.toCHTML(chtml);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
            }
            finally { if (e_1) throw e_1.error; }
        }
        var e_1, _c;
    };
    CHTMLWrapper.prototype.getBBox = function (save) {
        if (save === void 0) { save = true; }
        if (this.bboxComputed) {
            return this.bbox;
        }
        var bbox = (save ? this.bbox : BBox_js_1.BBox.zero());
        this.computeBBox(bbox);
        this.bboxComputed = save;
        return bbox;
    };
    CHTMLWrapper.prototype.computeBBox = function (bbox) {
        bbox.empty();
        try {
            for (var _a = __values(this.childNodes), _b = _a.next(); !_b.done; _b = _a.next()) {
                var child = _b.value;
                bbox.append(child.getBBox());
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
            }
            finally { if (e_2) throw e_2.error; }
        }
        bbox.clean();
        var e_2, _c;
    };
    CHTMLWrapper.prototype.getStyles = function () {
        var styleString = this.node.attributes.getExplicit('style');
        if (!styleString)
            return;
        this.styles = this.html('span').style;
        var style = this.styles;
        style.cssText = styleString;
        for (var i = 0, m = CHTMLWrapper.removeStyles.length; i < m; i++) {
            var id = CHTMLWrapper.removeStyles[i];
            if (style[id]) {
                if (!this.removedStyles)
                    this.removedStyles = {};
                this.removedStyles[id] = style[id];
                style[id] = '';
            }
        }
    };
    CHTMLWrapper.prototype.getVariant = function () {
        if (!this.node.isToken)
            return;
        var attributes = this.node.attributes;
        var variant = attributes.get('mathvariant');
        if (!attributes.getExplicit('mathvariant')) {
            var values = attributes.getList('fontfamily', 'fontweight', 'fontstyle');
            if (this.removedStyles) {
                var style = this.removedStyles;
                if (style.fontFamily)
                    values.family = style.fontFamily;
                if (style.fontWeight)
                    values.weight = style.fontWeight;
                if (style.fontStyle)
                    values.style = style.fontStyle;
            }
            if (values.fontfamily)
                values.family = values.fontfamily;
            if (values.fontweight)
                values.weight = values.fontweight;
            if (values.fontstyle)
                values.style = values.fontstyle;
            if (values.weight && values.weight.match(/^\d+$/)) {
                values.weight = (parseInt(values.weight) > 600 ? 'bold' : 'normal');
            }
            if (values.family) {
                variant = this.explicitVariant(values.family, values.weight, values.style);
            }
            else {
                if (this.node.getProperty('variantForm'))
                    variant = '-TeX-variant';
                variant = (CHTMLWrapper.BOLDVARIANTS[values.weight] || {})[variant] || variant;
                variant = (CHTMLWrapper.ITALICVARIANTS[values.style] || {})[variant] || variant;
            }
        }
        this.variant = variant;
    };
    CHTMLWrapper.prototype.explicitVariant = function (fontFamily, fontWeight, fontStyle) {
        var style = this.styles;
        if (!style)
            style = this.styles = this.html('span').style;
        style.fontFamily = fontFamily;
        if (fontWeight)
            style.fontWeight = fontWeight;
        if (fontStyle)
            style.fontStyle = fontStyle;
        return '-explicitFont';
    };
    CHTMLWrapper.prototype.getScale = function () {
        var scale = 1, parent = this.parent;
        var pscale = (parent ? parent.bbox.scale : 1);
        var attributes = this.node.attributes;
        var scriptlevel = Math.min(attributes.get('scriptlevel'), 2);
        var fontsize = attributes.get('fontsize');
        var mathsize = (this.node.isToken || this.node.isKind('mstyle') ?
            attributes.get('mathsize') : attributes.getInherited('mathsize'));
        if (scriptlevel !== 0) {
            scale = Math.pow(attributes.get('scriptsizemultiplier'), scriptlevel);
            var scriptminsize = this.length2em(attributes.get('scriptminsize'), .8, 1);
            if (scale < scriptminsize)
                scale = scriptminsize;
        }
        if (this.removedStyles && this.removedStyles.fontSize && !fontsize) {
            fontsize = this.removedStyles.fontSize;
        }
        if (fontsize && !mathsize) {
            mathsize = fontsize;
        }
        if (mathsize !== '1') {
            scale *= this.length2em(mathsize, 1, 1);
        }
        if (this.variant === '-explicitFont') {
            scale /= this.metrics.scale;
        }
        this.bbox.scale = scale;
        this.bbox.rscale = scale / pscale;
    };
    CHTMLWrapper.prototype.getSpace = function () {
        var space = this.node.texSpacing();
        if (space) {
            this.bbox.L = this.length2em(space);
        }
    };
    CHTMLWrapper.prototype.standardCHTMLnode = function (parent) {
        var chtml = this.createCHTMLnode(parent);
        this.handleStyles();
        this.handleVariant();
        this.handleScale();
        this.handleColor();
        this.handleSpace();
        this.handleAttributes();
        return chtml;
    };
    CHTMLWrapper.prototype.createCHTMLnode = function (parent) {
        var href = this.node.attributes.get('href');
        if (href) {
            parent = parent.appendChild(this.html('a', { href: href }));
        }
        this.chtml = parent.appendChild(this.html('mjx-' + this.node.kind));
        return this.chtml;
    };
    CHTMLWrapper.prototype.handleStyles = function () {
        if (!this.styles)
            return;
        var styles = this.styles.cssText;
        if (styles) {
            this.chtml.style.cssText = styles;
        }
    };
    CHTMLWrapper.prototype.handleVariant = function () {
        if (this.node.isToken && this.variant !== '-explicitFont') {
            this.chtml.className = (this.font.getVariant(this.variant) || this.font.getVariant('normal')).classes;
        }
    };
    CHTMLWrapper.prototype.handleScale = function () {
        this.setScale(this.chtml, this.bbox.rscale);
    };
    CHTMLWrapper.prototype.setScale = function (chtml, rscale) {
        var scale = (Math.abs(rscale - 1) < .001 ? 1 : rscale);
        if (chtml && scale !== 1) {
            var size = this.percent(scale);
            if (exports.FONTSIZE[size]) {
                chtml.setAttribute('size', exports.FONTSIZE[size]);
            }
            else {
                chtml.style.fontSize = size;
            }
        }
        return chtml;
    };
    CHTMLWrapper.prototype.handleSpace = function () {
        if (this.bbox.L) {
            var space = this.em(this.bbox.L);
            if (exports.SPACE[space]) {
                this.chtml.setAttribute('space', exports.SPACE[space]);
            }
            else {
                this.chtml.style.marginLeft = space;
            }
        }
    };
    CHTMLWrapper.prototype.handleColor = function () {
        var attributes = this.node.attributes;
        var mathcolor = attributes.getExplicit('mathcolor');
        var color = attributes.getExplicit('color');
        var mathbackground = attributes.getExplicit('mathbackground');
        var background = attributes.getExplicit('background');
        if (mathcolor || color) {
            this.chtml.style.color = mathcolor || color;
        }
        if (mathbackground || background) {
            this.chtml.style.backgroundColor = mathbackground || background;
        }
    };
    CHTMLWrapper.prototype.handleAttributes = function () {
        var attributes = this.node.attributes;
        var defaults = attributes.getAllDefaults();
        var skip = CHTMLWrapper.skipAttributes;
        try {
            for (var _a = __values(attributes.getExplicitNames()), _b = _a.next(); !_b.done; _b = _a.next()) {
                var name_1 = _b.value;
                if (skip[name_1] === false || (!(name_1 in defaults) && !skip[name_1] &&
                    typeof (this.chtml[name_1]) === 'undefined')) {
                    this.chtml.setAttribute(name_1, attributes.getExplicit(name_1));
                }
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
            }
            finally { if (e_3) throw e_3.error; }
        }
        if (attributes.get('class')) {
            this.chtml.classList.add(attributes.get('class'));
        }
        var e_3, _c;
    };
    CHTMLWrapper.prototype.core = function () {
        return this.CHTML.nodeMap.get(this.node.core());
    };
    CHTMLWrapper.prototype.coreMO = function () {
        return this.CHTML.nodeMap.get(this.node.coreMO());
    };
    CHTMLWrapper.prototype.getText = function () {
        var text = '';
        if (this.node.isToken) {
            try {
                for (var _a = __values(this.node.childNodes), _b = _a.next(); !_b.done; _b = _a.next()) {
                    var child = _b.value;
                    if (child instanceof MmlNode_js_1.TextNode) {
                        text += child.getText();
                    }
                }
            }
            catch (e_4_1) { e_4 = { error: e_4_1 }; }
            finally {
                try {
                    if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
                }
                finally { if (e_4) throw e_4.error; }
            }
        }
        return text;
        var e_4, _c;
    };
    CHTMLWrapper.prototype.canStretch = function (direction) {
        this.stretch = FontData_js_1.NOSTRETCH;
        if (this.node.isEmbellished) {
            var core = this.core();
            if (core && core.node !== this.node) {
                if (core.canStretch(direction)) {
                    this.stretch = core.stretch;
                }
            }
        }
        return this.stretch.dir !== 0;
    };
    CHTMLWrapper.prototype.drawBBox = function () {
        var bbox = this.getBBox();
        var box = this.html('mjx-box', { style: {
                opacity: .25, 'margin-left': this.em(-bbox.w - bbox.R)
            } }, [
            this.html('mjx-box', { style: {
                    height: this.em(bbox.h),
                    width: this.em(bbox.w),
                    'background-color': 'red'
                } }),
            this.html('mjx-box', { style: {
                    height: this.em(bbox.d),
                    width: this.em(bbox.w),
                    'margin-left': this.em(-bbox.w),
                    'vertical-align': this.em(-bbox.d),
                    'background-color': 'green'
                } })
        ]);
        var node = this.chtml || this.parent.chtml;
        node.parentNode.appendChild(box);
        node.style.backgroundColor = '#FFEE00';
    };
    CHTMLWrapper.prototype.percent = function (m) {
        return LENGTHS.percent(m);
    };
    CHTMLWrapper.prototype.em = function (m) {
        return LENGTHS.em(m);
    };
    CHTMLWrapper.prototype.px = function (m, M) {
        if (M === void 0) { M = -LENGTHS.BIGDIMEN; }
        return LENGTHS.px(m, M, this.metrics.em);
    };
    CHTMLWrapper.prototype.length2em = function (length, size, scale) {
        if (size === void 0) { size = 1; }
        if (scale === void 0) { scale = null; }
        if (scale === null) {
            scale = this.bbox.scale;
        }
        return LENGTHS.length2em(length, size, scale, this.metrics.em);
    };
    CHTMLWrapper.prototype.unicodeChars = function (text) {
        return string_js_1.unicodeChars(text);
    };
    CHTMLWrapper.prototype.char = function (n, escape) {
        if (escape === void 0) { escape = false; }
        return this.font.char(n, escape);
    };
    CHTMLWrapper.prototype.html = function (type, def, content) {
        if (def === void 0) { def = {}; }
        if (content === void 0) { content = []; }
        return this.factory.chtml.html(type, def, content);
    };
    CHTMLWrapper.prototype.text = function (text) {
        return this.factory.chtml.text(text);
    };
    CHTMLWrapper.prototype.mmlText = function (text) {
        return this.node.factory.create('text').setText(text);
    };
    CHTMLWrapper.prototype.mmlNode = function (kind, properties, children) {
        if (properties === void 0) { properties = {}; }
        if (children === void 0) { children = []; }
        return this.node.factory.create(kind, properties, children);
    };
    return CHTMLWrapper;
}(Wrapper_js_1.AbstractWrapper));
CHTMLWrapper.kind = 'unknown';
CHTMLWrapper.autoStyle = true;
CHTMLWrapper.styles = {
    'mjx-chtml [space="1"]': { 'margin-left': '.167em' },
    'mjx-chtml [space="2"]': { 'margin-left': '.222em' },
    'mjx-chtml [space="3"]': { 'margin-left': '.278em' },
    'mjx-chtml [size="s"]': { 'font-size': '70.7%' },
    'mjx-chtml [size="ss"]': { 'font-size': '50%' },
    'mjx-chtml [size="Tn"]': { 'font-size': '60%' },
    'mjx-chtml [size="sm"]': { 'font-size': '85%' },
    'mjx-chtml [size="lg"]': { 'font-size': '120%' },
    'mjx-chtml [size="Lg"]': { 'font-size': '144%' },
    'mjx-chtml [size="LG"]': { 'font-size': '173%' },
    'mjx-chtml [size="hg"]': { 'font-size': '207%' },
    'mjx-chtml [size="HG"]': { 'font-size': '249%' },
    'mjx-chtml [width="full"]': {
        width: '100%'
    },
    'mjx-box': { display: 'inline-block' },
    'mjx-block': { display: 'block' },
    'mjx-itable': { display: 'inline-table' },
    'mjx-row': { display: 'table-row' },
    'mjx-row > *': { display: 'table-cell' },
    'mjx-mn': { display: 'inline-block' },
    'mjx-mtext': { display: 'inline-block' },
    'mjx-merror': {
        display: 'inline-block',
        color: 'red',
        'background-color': 'yellow'
    },
    'mjx-mphantom': { visibility: 'hidden' }
};
CHTMLWrapper.removeStyles = [
    'fontSize', 'fontFamily', 'fontWeight',
    'fontStyle', 'fontVariant', 'font'
];
CHTMLWrapper.skipAttributes = {
    fontfamily: true, fontsize: true, fontweight: true, fontstyle: true,
    color: true, background: true,
    'class': true, href: true, style: true,
    xmlns: true
};
CHTMLWrapper.BOLDVARIANTS = {
    bold: {
        normal: 'bold',
        italic: 'bold-italic',
        fraktur: 'bold-fraktur',
        script: 'bold-script',
        'sans-serif': 'bold-sans-serif',
        'sans-serif-italic': 'sans-serif-bold-italic'
    },
    normal: {
        bold: 'normal',
        'bold-italic': 'italic',
        'bold-fraktur': 'fraktur',
        'bold-script': 'script',
        'bold-sans-serif': 'sans-serif',
        'sans-serif-bold-italic': 'sans-serif-italic'
    }
};
CHTMLWrapper.ITALICVARIANTS = {
    italic: {
        normal: 'italic',
        bold: 'bold-italic',
        'sans-serif': 'sans-serif-italic',
        'bold-sans-serif': 'sans-serif-bold-italic'
    },
    normal: {
        italic: 'normal',
        'bold-italic': 'bold',
        'sans-serif-italic': 'sans-serif',
        'sans-serif-bold-italic': 'bold-sans-serif'
    }
};
exports.CHTMLWrapper = CHTMLWrapper;
var _a;
//# sourceMappingURL=Wrapper.js.map